# Written by Bram Cohen
# see LICENSE.txt for license information
# Updated and modified for ABC_OKC : Old King Cole

from threading import Event
from BitTornado.utility import getABCUtility, getINIT_STATE


class DownloaderFeedback:
    def __init__(self, storagewrapper, choker, httpdl, sched, upfunc, downfunc,
            pexupfunc, leftfunc, file_length, finflag, sp, statistics, config,
            statusfunc, interval):
        self.storagewrapper = storagewrapper
        self.choker = choker
        self.httpdl = httpdl
        self.sched = sched
        self.upfunc = upfunc
        self.downfunc = downfunc
        self.pexupfunc = pexupfunc
        self.leftfunc = leftfunc
        self.file_length = file_length
        self.finflag = finflag
        self.sp = sp
        self.statistics = statistics
        self.config = config
        self.lastids = []
        self.doneprocessing = Event()
        self.doneprocessing.set()
        self.statusfunc = statusfunc
        self.interval = interval
        self.httpseedid = getABCUtility().lang.get('httpseed')
        self.httpseeddirection = getINIT_STATE()[1][0]
        self.autodisplay()

    def _rotate(self):
        cs = self.choker.connections
        for id in self.lastids:
            for i in xrange(len(cs)):
                if cs[i].get_id() == id:
                    return cs[i:] + cs[:i]
        return cs

    def spews(self):
        l = []
        cs = self._rotate()
        self.lastids = [c.get_id() for c in cs]
        for c in cs:
            u = c.upload
            d = c.download
            a = {}
            a['id'] = c.get_readable_id()
            a['ip'] = c.get_ip()
            a['optimistic'] = (c is self.choker.connections[0])
            a['direction'] = getINIT_STATE()[c.is_locally_initiated()][c.is_encrypted()]
            a['uprate'] = u.measure.get_rate()
            a['uinterested'] = u.is_interested()
            a['uchoked'] = u.is_choked()
            a['downrate'] = d.measure.get_rate()
            a['dinterested'] = d.is_interested()
            a['dchoked'] = d.is_choked()
            a['snubbed'] = d._check_snubbed()
            a['utotal'] = u.measure.get_total()
            a['dtotal'] = d.measure.get_total()
            if self.config['magnet']:
                a['completed'] = None
            elif d.gothavebitfield:
                if len(d.have) > 0:
                    a['completed'] = float(len(d.have) - d.have.nbzero) / len(d.have)
                else:
                    a['completed'] = 1.0
            else:
                a['completed'] = None
            if self.config['magnet']:
                a['speed'] = None
            elif d.have.complete():
                a['speed'] = 0.
            elif d.gothave:
                a['speed'] = d.peermeasure.get_rate()
            else:
                a['speed'] = None

            l.append(a)                                               

        for dl in self.httpdl.get_downloads():
            if dl.goodseed:
                a = {}
                a['id'] = self.httpseedid
                a['ip'] = dl.baseurl
                a['optimistic'] = False
                a['direction'] = self.httpseeddirection
                a['uprate'] = 0
                a['uinterested'] = False
                a['uchoked'] = False
                a['downrate'] = dl.measure.get_rate()
                a['dinterested'] = True
                a['dchoked'] = a['snubbed'] = not dl.active
                a['utotal'] = None
                a['dtotal'] = dl.measure.get_total()
                a['completed'] = 1.0
                a['speed'] = None

                l.append(a)

        return l

    def gather(self, displayfunc = None):
        s = {'stats': self.statistics.update()}
        if self.sp.isSet():
            s['spew'] = self.spews()
        else:
            s['spew'] = None
        s['up'] = self.upfunc()
        s['pexup'] = self.pexupfunc()
        if self.finflag.isSet():
            s['done'] = self.file_length
            return s
        s['down'] = self.downfunc()
        obtained, desired = self.leftfunc()
        s['done'] = obtained
        s['wanted'] = desired
        if self.storagewrapper.initfrac is not None:
            s['frac'] = self.storagewrapper.initfrac
            s['act'] = self.storagewrapper.initact
        elif desired > 0:
            s['frac'] = float(obtained) / desired
            s['act'] = None
        elif self.config['magnet']:
            s['frac'] = 0.0
            s['act'] = None
        else:
            s['frac'] = 1.0
            s['act'] = None
        return s

    def display(self):
        if not self.doneprocessing.isSet():
            return
        self.doneprocessing.clear()
        stats = self.gather()
        if self.finflag.isSet():
            self.statusfunc(dpflag = self.doneprocessing,
                upRate = stats['up'], pexUpRate = stats['pexup'],
                statistics = stats['stats'], spew = stats['spew'])
        else:
            self.statusfunc(dpflag = self.doneprocessing,
                fractionDone = stats['frac'], sizeDone = stats['done'],
                downRate = stats['down'], upRate = stats['up'],
                pexUpRate = stats['pexup'],
                activity = stats['act'], statistics = stats['stats'],
                spew = stats['spew'])

    def autodisplay(self):
        self.sched(self.autodisplay, self.interval)
        self.display()
