# Written by Bram Cohen
# see LICENSE.txt for license information
# Updated and modified for ABC_OKC : Old King Cole

from time import clock

from BitTornado.CurrentRateMeasure import Measure
from BitTornado.buffer import PIECEREADBUFFERPOOL, newSendPiecePartBuffer


class Upload:
    def __init__(self, connection, ratelimiter, choker, storage,
                 picker, piecereadbufferlength, config, sched):
        self.connection = connection
        self.ratelimiter = ratelimiter
        self.choker = choker
        self.storage = storage
        self.picker = picker
        self.piecereadbufferlength = piecereadbufferlength
        self.config = config
        self.sched = sched
        self.max_slice_length = config['max_slice_length']
        self.choked = True
        self.cleared = True
        self.interested = False
        self.super_seeding = False
        self.buffer = []
        self.measure = Measure(config['max_rate_period'])
        self.was_ever_interested = False
        if not self.config['magnet']:
            if storage.get_amount_left() == 0:
                if choker.super_seed:
                    self.super_seeding = True   # flag, and don't send bitfield
                    self.seed_have_list = []    # set from piecepicker
                    self.skipped_count = 0
                elif config['breakup_seed_bitfield']:
                    bitfield, msgs = storage.get_have_list_cloaked()
                    connection.send_bitfield(bitfield)
                    for have in msgs:
                        connection.send_have(have)
                else:
                    connection.send_bitfield(storage.get_have_list())
            elif storage.do_I_have_anything():
                connection.send_bitfield(storage.get_have_list())
        self.piecedl = None
        self.piecebuf = None
        self.piecebuflastaccess = 0
        self.read_buffer_expiration = config['read_buffer_expiration']

    def updateData(self):
        if self.storage.get_amount_left() == 0:
            if self.choker.super_seed:
                self.super_seeding = True   # flag, and don't send bitfield
                self.seed_have_list = []    # set from piecepicker
                self.skipped_count = 0
            elif config['breakup_seed_bitfield']:
                bitfield, msgs = self.storage.get_have_list_cloaked()
                self.connection.send_bitfield(bitfield)
                for have in msgs:
                    self.connection.send_have(have)
            else:
                self.connection.send_bitfield(self.storage.get_have_list())
        elif self.storage.do_I_have_anything():
            self.connection.send_bitfield(self.storage.get_have_list())

    def got_not_interested(self):
        if self.interested:
            self.interested = False
            del self.buffer[:]
            if self.piecebuf is not None:
                self.piecebuf.release()
                self.piecebuf = None
            self.choker.not_interested(self.connection)

    def got_interested(self):
        if not self.interested:
            self.interested = True
            self.was_ever_interested = True
            self.choker.interested(self.connection)

    def get_piece_from_buffer(self, begin, length):
        self.piecebuflastaccess = clock()
        return newSendPiecePartBuffer(self.piecebuf.viewSlice(begin, begin + length))

    def get_upload_chunk(self):
        if self.choked or not self.buffer:
            return None
        index, begin, length = self.buffer.pop(0)
        if self.piecebuf is None or index != self.piecedl:
            # Reading a new piece buffer is necessary
            if self.piecebuf is not None:
                self.piecebuf.release()
            self.piecebuf, err = self.storage.get_piece(index, 0, -1)
            if self.piecebuf is not None:
                # Read from piece buffer
                piece = self.get_piece_from_buffer(begin, length)
            elif err == 'overload':
                # Read from disc
                piecepartbuf, err = self.storage.get_piece(index, begin, length)
                if err:
                    piece = None
                else:
                    piece = newSendPiecePartBuffer(piecepartbuf.viewSlice())
                    piecepartbuf.release()
            else:
                piece = None
            self.piecedl = index
        else:
            # Keep on reading from piece buffer
            piece = self.get_piece_from_buffer(begin, length)
        if piece is None:
            self.connection.close()
            return None
        return (index, begin, piece)

    def got_request(self, index, begin, length):
        if ((self.super_seeding and not index in self.seed_have_list)
            or not self.interested or length > self.max_slice_length):
            self.connection.close()
            return
        if not self.cleared:
            self.buffer.append((index, begin, length))
        if not self.choked and self.connection.next_upload is None:
            self.ratelimiter.queue(self.connection)

    def got_cancel(self, index, begin, length):
        try:
            self.buffer.remove((index, begin, length))
        except ValueError:
            pass

    def choke(self):
        if not self.choked:
            self.choked = True
            self.connection.send_choke()
        if self.piecebuf is not None:
            self.piecebuf.release()
            self.piecebuf = None

    def choke_sent(self):
        del self.buffer[:]
        self.cleared = True

    def unchoke(self):
        if self.choked:
            if self.config['read_buffer_memory_saving'] == 0 \
               and PIECEREADBUFFERPOOL.isOverloaded(self.piecereadbufferlength):
                return
            self.choked = False
            self.cleared = False
            self.connection.send_unchoke()

    def disconnected(self):
        if self.piecebuf is not None:
            self.piecebuf.release()
            self.piecebuf = None

    def is_choked(self):
        return self.choked

    def is_interested(self):
        return self.interested

    def has_queries(self):
        return not self.choked and len(self.buffer) > 0

    def get_rate(self):
        return self.measure.get_rate()

    def expirePieceBuffer(self):
        if self.piecebuf is not None and clock() - self.piecebuflastaccess > self.read_buffer_expiration:
            self.piecebuf.release()
            self.piecebuf = None
