# Written by Bram Cohen
# see LICENSE.txt for license information
# Updated and modified for ABC_OKC : Old King Cole

from time import clock
from CurrentRateMeasure import Measure
from math import sqrt

try:
    sum([1])
except:
    sum = lambda a: reduce(lambda x, y: x + y, a, 0)

DEBUG = False

MAX_RATE_PERIOD = 10.0
MAX_RATE = 10e10


class RateLimiter:
    def __init__(self, sched, unitsize):
        self.sched = sched
        self.last = None
        self.unitsize = unitsize
        self.measure = Measure(MAX_RATE_PERIOD)
        self.upload_rate = MAX_RATE * 1024
        self.bytes_sent = 0
        self.last_time = self.emptyqueuetime = clock()

    def set_upload_rate(self, rate):
        if not rate:
            rate = MAX_RATE
        rate = max(int(rate * 1024), 1)
        if rate == self.upload_rate:
            return
        t = clock()
        self.bytes_sent -= (t - self.last_time) * self.upload_rate
        if self.last is None and (self.bytes_sent < -50000 or t - self.emptyqueuetime > 10):
            self.bytes_sent = 0
        self.last_time = t
        self.upload_rate = rate

    def queue(self, conn):
        assert conn.next_upload is None
        if self.last is None:
            self.last = conn
            conn.next_upload = conn
            self.try_send(True)
        else:
            conn.next_upload = self.last.next_upload
            self.last.next_upload = conn
            self.last = conn

    def try_send(self, check_time = False):
        t = clock()
        self.bytes_sent -= (t - self.last_time) * self.upload_rate
        if check_time and (self.bytes_sent < -50000 or t - self.emptyqueuetime > 10):
            self.bytes_sent = 0
        self.last_time = t
        cur = self.last.next_upload
        while self.bytes_sent <= 0:
            bytes = cur.send_partial(self.unitsize)
            if bytes:
                self.bytes_sent += bytes
                cur.upload.measure.update_rate(bytes)
                self.measure.update_rate(bytes)
            if bytes == 0 or cur.backlogged():
                if self.last is cur:
                    self.last = None
                    cur.next_upload = None
                    self.emptyqueuetime = t
                    break
                self.last.next_upload = cur.next_upload
                cur.next_upload = None
                cur = self.last.next_upload
            else:
                self.last = cur
                cur = cur.next_upload
        else:
            self.sched(self.try_send, max(self.bytes_sent / self.upload_rate, 0.0001))

    def adjust_sent(self, bytes):
        self.bytes_sent = min(self.bytes_sent + bytes, self.upload_rate * 3)
        self.measure.update_rate(bytes)
