# Written by Bram Cohen
# see LICENSE.txt for license information
# Updated and modified for ABC_OKC : Old King Cole

from SocketHandler import SocketHandler, UPnP_ERROR
from buffer import READSIZE
from utility import exceptionArgsToString, sysencoding, getABCUtility
import socket
from StringIO import StringIO
from traceback import print_exc
from select import error
from threading import Event
from time import clock
import sys


def autodetect_ipv6():
    try:
        assert sys.version_info >= (2, 3)
        assert socket.has_ipv6
        socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
    except:
        return 0
    return 1

def autodetect_socket_style():
    if sys.platform.find('linux') < 0:
        return 1
    try:
        f = open('/proc/sys/net/ipv6/bindv6only', 'r')
        dual_socket_style = int(f.read())
        f.close()
        return int(not dual_socket_style)
    except:
        return 0


class RawServer:
    def __init__(self, doneflag, timeout_check_interval, timeout, noisy = True,
                 ipv6_enable = True, failfunc = lambda x: None, errorfunc = None,
                 sockethandler = None, udp = False):
        self.timeout_check_interval = timeout_check_interval
        self.doneflag = doneflag
        self.noisy = noisy
        self.failfunc = failfunc
        self.errorfunc = errorfunc
        self.exccount = 0
        self.funcs = []
        self.externally_added = []
        self.udp = udp

        if sockethandler is None:
            self.sockethandler = SocketHandler(timeout, ipv6_enable, READSIZE, udp = self.udp)
        else:
            self.sockethandler = sockethandler

    def _add_task(self, func, time, args, id):
        i = 0
        for f in self.funcs:
            if f[0] > time:
                self.funcs.insert(i, (time, func, args, id))
                break
            i += 1
        else:
            self.funcs.append((time, func, args, id))

    def add_task(self, func, delay = 0, args = [], id = None):
        try:
            self.externally_added.append((func, delay, args, id))
        except:
            pass

    def add_task_prio(self, func, delay = 0, args = [], id = None):
        try:
            self.externally_added.insert(0, (func, delay, args, id))
        except:
            pass

    def scan_for_timeouts(self):
        self.add_task(self.scan_for_timeouts, self.timeout_check_interval)
        self.sockethandler.scan_for_timeouts()

    def bind(self, port, bind = '', reuse = False,
             ipv6_socket_style = 1, upnp = False):
        return self.sockethandler.bind(port, bind, reuse, ipv6_socket_style, upnp)

    def find_and_bind(self, minport, maxport, bind = '', reuse = False,
                      ipv6_socket_style = 1, upnp = 0, randomizer = False):
        return self.sockethandler.find_and_bind(minport, maxport, bind, reuse,
                                                ipv6_socket_style, upnp, randomizer)

    def start_connection_raw(self, dns, socktype, handler = None):
        return self.sockethandler.start_connection_raw(dns, socktype, handler)

    def start_connection(self, dns, handler = None, randomize = False):
        return self.sockethandler.start_connection(dns, handler, randomize)

    def get_stats(self):
        return self.sockethandler.get_stats()

    def pop_external(self):
        clck = clock()
        while self.externally_added:
            a, b, c, d = self.externally_added.pop(0)
            self._add_task(a, b + clck, c, d)

    def listen_forever(self, handler):
        self.sockethandler.set_handler(handler)
        if self.funcs is None:
            self.funcs = []
        if self.externally_added is None:
            self.externally_added = []
        if not self.udp:
            self.add_task(self.scan_for_timeouts, self.timeout_check_interval)
        try:
            while not self.doneflag.isSet():
                try:
                    self.pop_external()
                    events = self.sockethandler.do_poll(0.01)
                    if self.doneflag.isSet():
                        return False
                    while self.funcs and self.funcs[0][0] <= clock():
                        garbage, func, args, id = self.funcs.pop(0)
                        try:
                            func(*args)
                        except SystemError, e:
                            self.failfunc(getABCUtility().lang.get('systemerror') + ' - ' + exceptionArgsToString(e))
                            return True
                        except MemoryError, e:
                            self.failfunc(getABCUtility().lang.get('memoryerror') + ' - ' + exceptionArgsToString(e))
                            return True
                        except IOError, e:
                            self.failfunc(getABCUtility().lang.get('ioerror') + ' - ' + exceptionArgsToString(e))
                            return True
                        except:
                            if self.noisy and self.errorfunc:
                                self.exception()
                    func = None
                    args = None
                    self.sockethandler.close_dead()
                    self.sockethandler.handle_events(events)
                    if self.doneflag.isSet():
                        return False
                    self.sockethandler.close_dead()
                except SystemError, e:
                    self.failfunc(getABCUtility().lang.get('systemerror') + ' - ' + exceptionArgsToString(e))
                    return True
                except MemoryError, e:
                    self.failfunc(getABCUtility().lang.get('memoryerror') + ' - ' + exceptionArgsToString(e))
                    return True
                except IOError, e:
                    self.failfunc(getABCUtility().lang.get('ioerror') + ' - ' + exceptionArgsToString(e))
                    return True
                except error:
                    if self.doneflag.isSet():
                        return True
                except:
                    if self.errorfunc:
                        self.exception()
                        if self.exccount > 10:
                            return True
        finally:
            self.externally_added = None
            self.funcs = None

    def exception(self):
        self.exccount += 1
        data = StringIO()
        print_exc(file = data)
        self.errorfunc(data.getvalue().decode(sysencoding))

    def shutdown(self):
        self.sockethandler.shutdown()
