##############################################################################
# Module : bitfield.py
# Author : Old King Cole
# Date   : 07/11/2007
#
# Description : Bitfields
#
##############################################################################
class Bitfield:
    def __init__(self, nbbit = None, string = None, boollist = None, copyfrom = None, rowlength = 10):
        # Can be created with "nbbit" zero bits
        # or initialized with the "nbbit" first bits from a string "string"
        # or initialized from a list of booleans "boollist"
        # or duplicated from an existing Bitfield "copyfrom"
        if copyfrom is not None:
            self.nbbit = copyfrom.nbbit
            self.rowlength = copyfrom.rowlength
            self.field = []
            for r in copyfrom.field:
                self.field.append(r[:])
            self.nbzero = copyfrom.nbzero
        else:
            if boollist is not None:
                nbbit = len(boollist)
            elif nbbit is None:
                raise ValueError, "number of bits is mandatory"
            self.nbbit = nbbit
            self.rowlength = rowlength
            self.field = []
            nbchar, lastchar = divmod(nbbit, 8)
            if lastchar:
                nbchar += 1
            nbrow, lastrow = divmod(nbchar, rowlength)
            self.nbzero = nbbit
            if boollist is None:
                if string is None:
                    for row in xrange(nbrow):
                        self.field.append('\x00' * rowlength)
                    if lastrow:
                        self.field.append('\x00' * lastrow)
                else:
                    nbbitinstring = len(string) << 3
                    if nbbit < nbbitinstring - 7 or nbbit > nbbitinstring:
                        raise ValueError, "number of bits does not fit string length"
                    for rowhead in xrange(0, nbrow * rowlength, rowlength):
                        self.field.append(string[rowhead:rowhead + rowlength])
                    if lastrow:
                        self.field.append(string[-lastrow:])
                    # Check if extra bits of last byte are null
                    if nbbitinstring != nbbit and (ord(string[-1]) & 2 ** (nbbitinstring - nbbit) - 1):
                        raise ValueError, "extra bits are not null"
                    self.nbzero -= sum(self)
            else:
                for row in xrange(nbrow):
                    self.field.append('')
                    rowhead = row * rowlength
                    for col in xrange(rowlength):
                        coord = (rowhead + col) << 3
                        val = 0
                        for i in xrange(8):
                            boolnumber = coord + i
                            if boolnumber < nbbit and boollist[boolnumber]:
                                val += 1 << (7 - i)
                                self.nbzero -= 1
                        self.field[row] += chr(val)
                if lastrow:
                    self.field.append('')
                    rowhead = nbrow * rowlength
                    for col in xrange(lastrow):
                        coord = (rowhead + col) << 3
                        val = 0
                        for i in xrange(8):
                            boolnumber = coord + i
                            if boolnumber < nbbit and boollist[boolnumber]:
                                val += 1 << (7 - i)
                                self.nbzero -= 1
                        self.field[nbrow] += chr(val)

    def getCoordinates(self, index):
        if index >= self.nbbit or index < 0:
            raise IndexError, "index out of range"
        nbchar, bit = divmod(index, 8)
        row, char = divmod(nbchar, self.rowlength)
        return row, char, bit

    def __setitem__(self, index, value):
        if index < 0:
            index = self.nbbit + index
        row, char, bit = self.getCoordinates(index)
        oldvalue = self.__getitem__(index)
        if value:
            if not oldvalue:
                newchar = chr(ord(self.field[row][char]) | 1 << (7 - bit))
                self.nbzero -= 1
                self.field[row] = self.field[row][:char] + newchar + self.field[row][char + 1:]
        elif oldvalue:
            newchar = chr(ord(self.field[row][char]) & ~(1 << (7 - bit)))
            self.nbzero += 1
            self.field[row] = self.field[row][:char] + newchar + self.field[row][char + 1:]

    def __getitem__(self, index):
        if index < 0:
            index = self.nbbit + index
        row, char, bit = self.getCoordinates(index)
        return (ord(self.field[row][char]) >> (7 - bit)) & 1

    def __len__(self):
        return self.nbbit

    def __repr__(self):
        repr = ''
        for r in self.field:
            for c in r:
                ordc = ord(c)
                reprchar = ''
                for i in xrange(8):
                    ordc, r = divmod(ordc, 2)
                    reprchar = str(r) + reprchar
                repr += reprchar
        return repr[:self.nbbit]

    def tostring(self):
        return ''.join(self.field)

    def complete(self):
        return not self.nbzero
