# Written by John Hoffman
# see LICENSE.txt for license information
# Updated and modified for ABC_OKC : Old King Cole

import socket
from httplib import HTTPConnection, HTTPSConnection, HTTPException
from urlparse import urlparse
from gzip import GzipFile
from StringIO import StringIO
from random import getrandbits
from binascii import b2a_hex
from time import clock
from threading import Semaphore

from bencode import bdecode
from utility import exceptionArgsToString, getABCUtility

from __init__ import USERAGENT

MAX_REDIRECTS = 10
UNRESOLVED = {}
RESOLVESEM = Semaphore(1)

def toint(s):
    return long(b2a_hex(s), 16)

def tobinary(i):
    return (chr(i >> 24) + chr((i >> 16) & 0xFF) + 
            chr((i >> 8) & 0xFF) + chr(i & 0xFF))


class urlopen:
    def __init__(self, url):
        self.tries = 0
        self.gzipfile = None
        self._open(url.strip())

    def _open(self, url):
        self.tries += 1
        if self.tries > MAX_REDIRECTS:
            raise IOError, (getABCUtility().lang.get('httperror'), 500,
                            getABCUtility().lang.get('redirrecursion'))
        up = urlparse(url)
        if up.scheme != 'http' and up.scheme != 'https':
            raise IOError, (getABCUtility().lang.get('urlerror'), getABCUtility().lang.get('unknownurltype'), up.scheme, url)

        RESOLVESEM.acquire()
        lastunresolved = UNRESOLVED.get(up.hostname)
        now = clock()
        if lastunresolved and now - lastunresolved < 300:
            RESOLVESEM.release()
            raise IOError, (getABCUtility().lang.get('urlerror'), getABCUtility().lang.get('delayedresolving') + up.hostname)
        ip = getABCUtility().getHostByName(up.hostname)
        if ip is None:
            UNRESOLVED[up.hostname] = now
            RESOLVESEM.release()
            raise IOError, (getABCUtility().lang.get('urlerror'), getABCUtility().lang.get('cantresolve') + up.hostname)
        if lastunresolved:
            del UNRESOLVED[up.hostname]
        RESOLVESEM.release()
        url = up.path
        if up.params:
            url += ';' + up.params
        if up.query:
            url += '?' + up.query
        try:
            if up.scheme == 'http':
                self.connection = HTTPConnection(up.netloc, timeout = 30)
            else:
                self.connection = HTTPSConnection(up.netloc, timeout = 30)
            self.connection.request('GET', url, None,
                                    {'User-Agent': USERAGENT[0],
                                     'Accept-Encoding': 'gzip'})
            self.response = self.connection.getresponse()
        except Exception, e:
            raise IOError, (getABCUtility().lang.get('httperror'), exceptionArgsToString(e))
        status = self.response.status
        if status == 301 or status == 302:
            try:
                self.connection.close()
            except:
                pass
            self._open(self.response.getheader('Location'))
        elif status != 200:
            try:
                self.connection.close()
            except:
                pass
            raise IOError, (getABCUtility().lang.get('httperror'), status, self.response.reason)

    def read(self):
        try:
            data = self.response.read()
        except:
            raise IOError, (getABCUtility().lang.get('httperror'), getABCUtility().lang.get('corruptresponse'))
        if self.response.getheader('Content-Encoding', '').find('gzip') >= 0:
            try:
                self.gzipfile = GzipFile(fileobj = StringIO(data))
                data = self.gzipfile.read()
            except:
                raise IOError, (getABCUtility().lang.get('httperror'), getABCUtility().lang.get('corruptresponse'))
        return data

    def close(self):
        self.connection.close()
        if self.gzipfile:
            self.gzipfile.close()


def udpurlopen(url, request):
    # Max timeout for each request is 30 s
    up = urlparse(url.strip())
    if up.port is None:
        raise IOError, (getABCUtility().lang.get('urlerror'), getABCUtility().lang.get('noport'))

    RESOLVESEM.acquire()
    lastunresolved = UNRESOLVED.get(up.hostname)
    now = clock()
    if lastunresolved and now - lastunresolved < 300:
        RESOLVESEM.release()
        raise IOError, (getABCUtility().lang.get('urlerror'), getABCUtility().lang.get('delayedresolving') + up.hostname)
    ip = getABCUtility().getHostByName(up.hostname)
    if ip is None:
        UNRESOLVED[up.hostname] = now
        RESOLVESEM.release()
        raise IOError, (getABCUtility().lang.get('urlerror'), getABCUtility().lang.get('cantresolve') + up.hostname)
    if lastunresolved:
        del UNRESOLVED[up.hostname]
    RESOLVESEM.release()
    # Get connection ID
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.settimeout(15)
    reqnumber = 0
    while True:
        reqnumber += 1
        transactionid = tobinary(getrandbits(32))
        sock.sendto("\x00\x00\x04\x17'\x10\x19\x80" + tobinary(0) + transactionid, 0, (ip, up.port))
        try:
            data = sock.recv(512)
        except socket.timeout:
            if reqnumber < 2:
                continue
            sock.close()
            raise IOError, (getABCUtility().lang.get('udperror'), getABCUtility().lang.get('timeout'))
        break
    if len(data) < 16:
        sock.close()
        raise IOError, (getABCUtility().lang.get('udperror'), getABCUtility().lang.get('corruptresponse'))
    receivedaction = toint(data[0:4])
    receivedtransactionid = data[4:8]
    if receivedtransactionid == transactionid and receivedaction == 3:
        sock.close()
        raise IOError, (getABCUtility().lang.get('udperror'), data[8:])
    if receivedtransactionid != transactionid or receivedaction != 0:
        sock.close()
        raise IOError, (getABCUtility().lang.get('udperror'), getABCUtility().lang.get('corruptresponse'))
    connectionid = data[8:16]

    # Send request
    for reqnumber in xrange(2):
        transactionid = tobinary(getrandbits(32))
        sock.sendto(connectionid + tobinary(request[0]) + transactionid + request[1], 0, (ip, up.port))
        try:
            data = sock.recv(4096)
        except socket.timeout:
            if reqnumber == 0:
                continue
            sock.close()
            raise IOError, (getABCUtility().lang.get('udperror'), getABCUtility().lang.get('timeout'))
        break
    sock.close()
    receivedaction = toint(data[0:4])
    receivedtransactionid = data[4:8]
    if receivedtransactionid == transactionid and receivedaction == 3:
        raise IOError, (getABCUtility().lang.get('udperror'), data[8:])
    if receivedtransactionid != transactionid or receivedaction != request[0]:
        raise IOError, (getABCUtility().lang.get('udperror'), getABCUtility().lang.get('corruptresponse'))

    return data
