##############################################################################
# Module : torrentparamsdlg.py
# Author :
# Date   :
# Updated and modified for ABC_OKC : Old King Cole
#
# Description : Dialog for editing torrent parameters
#
##############################################################################
import sys, wx
from string import whitespace


class TorrentParametersDialog(wx.Dialog):
    def __init__(self, parent, ID, torrent, queue, panel, multitorrent, dht,
                 pos = wx.DefaultPosition, size = wx.DefaultSize,
                 style = wx.DEFAULT_DIALOG_STYLE):
        self.utility = parent.utility
        self.localize = self.utility.lang.get
        self.multitorrent = multitorrent
        if multitorrent:
            title = self.localize('torrentsettingmulti')
            self.firstvertgap = 4
        else:
            title = self.localize('torrentsetting')
            self.firstvertgap = 0
        self.parent = parent
        self.panel = panel
        self.torrent = torrent
        pre = wx.PreDialog()
        pre.Create(parent, ID, title + self.torrent.name, pos, size, style)
        self.this = pre.this
        self.queue = queue

        outerbox = wx.BoxSizer(wx.VERTICAL)

        self.notebook = wx.Notebook(self, -1)
        self.notebook.SetPadding((4, 4))
        self.notebook.AddPage(self.makeLimitsPanel(self.notebook), self.localize('torlimits'))
        self.downloadrate.SetValue(str(torrent.maxlocaldownloadrate))
        self.prioritizedown.SetValue(torrent.prioritizedown)
        self.maxuploadspin.SetValue(int(torrent.maxupload))
        self.maxupload.SetValue(torrent.maxupload)
        self.uploadrate.SetValue(str(torrent.maxlocaluploadrate))
        self.prioritizeup.SetValue(torrent.prioritizeup)
        self.rb[int(torrent.uploadoption)].SetValue(True)
        self.cbhtime.SetValue(torrent.uploadtimeh)
        self.cbmtime.SetValue(torrent.uploadtimem)
        self.cbratio.SetValue(torrent.uploadratio)
        self.uploadtillcopycomplete.SetValue(torrent.uploadtillcopycomplete)
        self.notebook.AddPage(self.makeTimeOutPanel(self.notebook), self.localize('tortimeout'))
        self.timeoutswitch.SetValue(torrent.timeoutswitch)
        self.timeoutaction.SetSelection(torrent.timeoutaction)
        self.timeoutwork.SetValue(torrent.timeoutwork)
        self.timeoutworkdown.SetValue(torrent.timeoutworkdown)
        self.timeoutseed.SetValue(torrent.timeoutseed)
        self.timeoutseedup.SetValue(torrent.timeoutseedup)
        if multitorrent or not dht or torrent.private:
            self.timeoutworktrack.SetValue(torrent.timeoutworktrack)
            self.timeoutseedtrack.SetValue(torrent.timeoutseedtrack)
        else:
            self.timeoutworktrack.Disable()
            self.timeoutseedtrack.Disable()
        self.cb_tracker.SetValue(torrent.timeouttracker)
        self.cb_download.SetValue(torrent.timeoutdownload)
        self.cb_upload.SetValue(torrent.timeoutupload)
        self.notebook.AddPage(self.makeLabMovTraPanel(self.notebook), self.localize('torlabmovtra'))
        self.shortlabel.SetValue(torrent.shortlabel)
        self.movefolder.SetValue(torrent.movefolder)
        self.exttracker.SetValue(torrent.exttracker)
        self.exttrackerurl.SetValue(torrent.exttrackerurl)

        if multitorrent:
            self.downloadrateswitch.SetValue(True)
            self.prioritizedownswitch.SetValue(True)
            self.maxuploadswitch.SetValue(True)
            self.uploadrateswitch.SetValue(True)
            self.prioritizeupswitch.SetValue(True)
            if self.utility.abcparams['multiparamdefault'] == '1':
                self.downloadswitch.SetValue(True)
                self.uploadswitch.SetValue(True)
                self.continueswitch.SetValue(True)
                self.timeoutsettingswitch.SetValue(True)
                self.timeoutvaluesswitch.SetValue(True)
                self.labelswitch.SetValue(True)
                self.movefolderswitch.SetValue(True)
                self.exttrackerswitch.SetValue(True)

        self.notebook.SetAutoLayout(True)

        self.maxpage = self.notebook.GetPageCount() - 1
        self.notebook.SetSelection(min(int(self.utility.abcparams['lastopentortab']), self.maxpage))

        outerbox.Add(self.notebook, 0, wx.EXPAND | wx.LEFT | wx.TOP | wx.RIGHT, 5)

        applybtn = wx.Button(self, -1, self.localize('apply'), size = (60, -1))
        okbtn = wx.Button(self, -1, self.localize('ok'), size = (60, -1))
        cancelbtn = wx.Button(self, -1, self.localize('cancel'), size = (60, -1))

        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        buttonbox.Add(applybtn, 0, wx.LEFT, 0)
        buttonbox.Add(okbtn, 0, wx.LEFT, 20)
        buttonbox.Add(cancelbtn, 0, wx.LEFT, 20)

        outerbox.Add(buttonbox, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.BOTTOM | wx.TOP, 6)

        self.SetAutoLayout(True)
        self.SetSizer(outerbox)
        self.Fit()

        self.utility.frameWindow(self.rb4, xl = 10, xb = -1, border = 1, midline = 1)

        # Add events
        self.Bind(wx.EVT_BUTTON, self.onOK, okbtn)
        self.Bind(wx.EVT_BUTTON, self.onApply, applybtn)
        self.Bind(wx.EVT_BUTTON, self.onCancel, cancelbtn)
        self.Bind(wx.EVT_CLOSE, self.onCancel)
        self.Bind(wx.EVT_MOUSEWHEEL, self.onMouseWheel)
        self.shortlabel.Bind(wx.EVT_CHAR, self.utility.onWriteFilteredText)
        self.movefolder.Bind(wx.EVT_CHAR, self.utility.onWriteFilteredText)
        self.notebook.Bind(wx.EVT_LEFT_DOWN, self.onLeftNoteBook)

        if self.utility.abcparams['torrentparamsdlgx'] != '-1':
            self.MoveXY(int(self.utility.abcparams['torrentparamsdlgx']), int(self.utility.abcparams['torrentparamsdlgy']))

    # Group Upload rate setting
    ###########################
    def makeLimitsPanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        downloadsection = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('downloadsetting')), wx.VERTICAL)
        if self.multitorrent:
            self.downloadswitch = wx.CheckBox(p, -1, '')
            downloadsection.Add(self.downloadswitch, 0, wx.ALIGN_RIGHT | wx.TOP, -17)

        maxdownloadratebox = wx.BoxSizer(wx.HORIZONTAL)
        maxdownloadratebox.Add(wx.StaticText(p, -1, self.localize('maxdownloadrate')), 0, wx.ALIGN_CENTER_VERTICAL)
        self.downloadrate = wx.TextCtrl(p, -1, '', wx.Point(-1, -1), wx.Size(40, -1))
        self.downloadrate.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        self.downloadrate.SetToolTipString(self.localize('0isnolimittt'))
        maxdownloadratebox.Add(self.downloadrate, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        maxdownloadratebox.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL)
        if self.multitorrent:
            maxdownloadratebox.Add((0, 0), 1)
            self.downloadrateswitch = wx.CheckBox(p, -1, '', size = (10, -1))
            maxdownloadratebox.Add(self.downloadrateswitch, 0, wx.ALIGN_CENTER_VERTICAL)

        downloadsection.Add(maxdownloadratebox, 0, wx.ALIGN_CENTER_VERTICAL | wx.EXPAND | wx.TOP, self.firstvertgap)

        prioritizedownbox = wx.BoxSizer(wx.HORIZONTAL)
        self.prioritizedown = wx.CheckBox(p, -1, self.localize('prioritizedown'))
        prioritizedownbox.Add(self.prioritizedown, 0, wx.ALIGN_CENTER_VERTICAL)
        if self.multitorrent:
            prioritizedownbox.Add((0, 0), 1)
            self.prioritizedownswitch = wx.CheckBox(p, -1, '', size = (10, -1))
            prioritizedownbox.Add(self.prioritizedownswitch, 0, wx.ALIGN_CENTER_VERTICAL)

        downloadsection.Add(prioritizedownbox, 0, wx.EXPAND | wx.TOP, 3)

        uploadsection = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('uploadsetting')), wx.VERTICAL)
        if self.multitorrent:
            self.uploadswitch = wx.CheckBox(p, -1, '')
            uploadsection.Add(self.uploadswitch, 0, wx.ALIGN_RIGHT | wx.TOP, -17)

        self.maxupload = wx.TextCtrl(p, -1, '', wx.Point(-1, -1), wx.Size(30, -1), wx.TE_READONLY | wx.TE_RIGHT)
        h = self.maxupload.GetSize().height
        self.maxuploadspin = wx.SpinButton(p, -1, wx.Point(-1, -1), wx.Size(h * 2 / 3, h), wx.SP_VERTICAL)
        self.maxuploadspin.SetRange(self.utility.minuploads, 100)
        self.Bind(wx.EVT_SPIN, self.onMaxUploadSpin, self.maxuploadspin)

        maxuploadbox = wx.BoxSizer(wx.HORIZONTAL)
        maxuploadbox.Add(wx.StaticText(p, -1, self.localize('maxuploads')), 0, wx.ALIGN_CENTER_VERTICAL)
        maxuploadbox.Add(self.maxupload, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        maxuploadbox.Add(self.maxuploadspin, 0, wx.ALIGN_CENTER_VERTICAL)
        if self.multitorrent:
            maxuploadbox.Add((0, 0), 1)
            self.maxuploadswitch = wx.CheckBox(p, -1, '', size = (10, -1))
            maxuploadbox.Add(self.maxuploadswitch, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_RIGHT)

        uploadsection.Add(maxuploadbox, 0, wx.ALIGN_CENTER_VERTICAL | wx.EXPAND | wx.TOP, self.firstvertgap)

        maxuploadratebox = wx.BoxSizer(wx.HORIZONTAL)
        maxuploadratebox.Add(wx.StaticText(p, -1, self.localize('maxuploadrate')), 0, wx.ALIGN_CENTER_VERTICAL)
        self.uploadrate = wx.TextCtrl(p, -1, '', wx.Point(-1, -1), wx.Size(40, -1))
        self.uploadrate.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        self.uploadrate.SetToolTipString(self.localize('0isnolimittt'))
        maxuploadratebox.Add(self.uploadrate, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        maxuploadratebox.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL)
        if self.multitorrent:
            maxuploadratebox.Add((0, 0), 1)
            self.uploadrateswitch = wx.CheckBox(p, -1, '', size = (10, -1))
            maxuploadratebox.Add(self.uploadrateswitch, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_RIGHT)

        uploadsection.Add(maxuploadratebox, 0, wx.ALIGN_CENTER_VERTICAL | wx.EXPAND | wx.TOP, 3)

        prioritizeupbox = wx.BoxSizer(wx.HORIZONTAL)
        self.prioritizeup = wx.CheckBox(p, -1, self.localize('prioritizeup'))
        prioritizeupbox.Add(self.prioritizeup, 0, wx.ALIGN_CENTER_VERTICAL)
        if self.multitorrent:
            prioritizeupbox.Add((0, 0), 1)
            self.prioritizeupswitch = wx.CheckBox(p, -1, '', size = (10, -1))
            prioritizeupbox.Add(self.prioritizeupswitch, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_RIGHT)

        uploadsection.Add(prioritizeupbox, 0, wx.EXPAND | wx.TOP, 3)

        # Upload setting for completed file
        continuesection = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('uploadoptforcompletedfile')), wx.VERTICAL)
        if self.multitorrent:
            self.continueswitch = wx.CheckBox(p, -1, '')
            continuesection.Add(self.continueswitch, 0, wx.ALIGN_RIGHT | wx.TOP, -17)

        uploadlist = [self.localize('unlimitedupload'), self.localize('continueuploadfor'), self.localize('untilratio'), self.localize('onfirstreached')]

        rb1 = wx.RadioButton(p, -1, uploadlist[0], wx.Point(-1, -1), wx.Size(-1, -1), wx.RB_GROUP)
        rb2 = wx.RadioButton(p, -1, uploadlist[1], wx.Point(-1, -1), wx.Size(-1, -1))
        rb3 = wx.RadioButton(p, -1, uploadlist[2], wx.Point(-1, -1), wx.Size(-1, -1))
        self.rb4 = wx.RadioButton(p, -1, uploadlist[3], wx.Point(-1, -1), wx.Size(-1, -1))
        self.rb = [rb1, rb2, rb3, self.rb4]

        mtimeval = ['0', '15', '30', '45']
        htimeval = [str(i) for i in xrange(25)]
        htimeval.extend(['48', '72', '96', '120', '144', '168', '336', '504', '672'])

        self.cbhtime = wx.ComboBox(p, -1, '', wx.Point(-1, -1),
                                   wx.Size(-1, -1), htimeval, wx.CB_DROPDOWN | wx.CB_READONLY)
        self.cbmtime = wx.ComboBox(p, -1, '', wx.Point(-1, -1),
                                   wx.Size(-1, -1), mtimeval, wx.CB_DROPDOWN | wx.CB_READONLY)

        continuesection.Add(rb1, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP, self.firstvertgap)

        time_sizer = wx.BoxSizer(wx.HORIZONTAL)
        time_sizer.Add(rb2, 0, wx.ALIGN_CENTER_VERTICAL)
        time_sizer.Add(self.cbhtime, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        time_sizer.Add(wx.StaticText(p, -1, self.localize('hour_short')), 0, wx.ALIGN_CENTER_VERTICAL)
        time_sizer.Add(self.cbmtime, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        time_sizer.Add(wx.StaticText(p, -1, self.localize('minute_short')), 0, wx.ALIGN_CENTER_VERTICAL)

        continuesection.Add(time_sizer, 0, wx.ALIGN_CENTER_VERTICAL)

        continuesection.Add(self.rb4, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 19)

        ratioval = ['10', '20', '30', '40', '50', '60', '70', '75', '80', '90', '100', '125', '150', '175', '200', '300', '400', '500', '750', '1000']
        self.cbratio = wx.ComboBox(p, -1, '', wx.Point(-1, -1), wx.Size(-1, -1), ratioval, wx.CB_DROPDOWN | wx.CB_READONLY)

        percent_sizer = wx.BoxSizer(wx.HORIZONTAL)
        percent_sizer.Add(rb3, 0, wx.ALIGN_CENTER_VERTICAL)
        percent_sizer.Add(self.cbratio, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        percent_sizer.Add(wx.StaticText(p, -1, "%"), 0, wx.ALIGN_CENTER_VERTICAL)

        continuesection.Add(percent_sizer, 0, wx.ALIGN_CENTER_VERTICAL)

        self.uploadtillcopycomplete = wx.CheckBox(p, -1, self.localize('uploadtillcopycomplete'))

        continuesection.Add(self.uploadtillcopycomplete, 0, wx.TOP, 3)

        sizer.Add(downloadsection, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(uploadsection, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(continuesection, 0, wx.EXPAND | wx.TOP, 3)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)
        return p

    # Group Timeout setting
    ##########################
    def makeTimeOutPanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        timeoutsettingbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('timeoutsetting')), wx.VERTICAL)
        if self.multitorrent:
            self.timeoutsettingswitch = wx.CheckBox(p, -1, '')
            timeoutsettingbox.Add(self.timeoutsettingswitch, 0, wx.ALIGN_RIGHT | wx.TOP, -17)

        # Global switch for timeouts
        self.timeoutswitch = wx.CheckBox(p, -1, self.localize('timeoutglobal'))
        timeoutsettingbox.Add(self.timeoutswitch, 0, wx.TOP, self.firstvertgap)

        timeoutactionbox = wx.BoxSizer(wx.HORIZONTAL)
        self.timeoutaction = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1),
                                         [self.localize('timeoutactionstandby'), self.localize('timeoutactionqueue'), self.localize('timeoutactionstop')],
                                         wx.CB_DROPDOWN | wx.CB_READONLY)
        timeoutactionbox.Add(self.timeoutaction, 0, wx.ALIGN_CENTER_VERTICAL)
        timeoutactionbox.Add(wx.StaticText(p, -1, self.localize('timeoutaction')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        timeoutsettingbox.Add(timeoutactionbox, 0, wx.TOP, 3)

        workseedbox = wx.BoxSizer(wx.HORIZONTAL)

        workbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, ''), wx.VERTICAL)

        timeoutworkbox = wx.BoxSizer(wx.HORIZONTAL)
        timeoutworkbox.Add(wx.StaticText(p, -1, ' '))
        self.timeoutwork = wx.CheckBox(p, -1, self.localize('timeoutwork'))
        timeoutworkbox.Add(self.timeoutwork)
        workbox.Add(timeoutworkbox, 0, wx.TOP, -9)
        workbox.Add(wx.StaticText(p, -1, self.localize('watch')), 0, wx.TOP | wx.BOTTOM, 3)
        timeoutworkwatch = wx.BoxSizer(wx.VERTICAL)
        self.timeoutworkdown = wx.CheckBox(p, -1, self.localize('timeoutdown'))
        timeoutworkwatch.Add(self.timeoutworkdown)
        self.timeoutworktrack = wx.CheckBox(p, -1, self.localize('timeouttrack'))
        timeoutworkwatch.Add(self.timeoutworktrack, 0, wx.TOP, 1)
        workbox.Add(timeoutworkwatch, 0, wx.LEFT, 15)
        workseedbox.Add(workbox, 1, wx.EXPAND)

        seedbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, ''), wx.VERTICAL)

        timeoutseedbox = wx.BoxSizer(wx.HORIZONTAL)
        timeoutseedbox.Add(wx.StaticText(p, -1, ' '))
        self.timeoutseed = wx.CheckBox(p, -1, self.localize('timeoutseed'))
        timeoutseedbox.Add(self.timeoutseed)
        seedbox.Add(timeoutseedbox, 0, wx.TOP, -9)
        seedbox.Add(wx.StaticText(p, -1, self.localize('watch')), 0, wx.TOP | wx.BOTTOM, 3)
        timeoutseedwatch = wx.BoxSizer(wx.VERTICAL)
        self.timeoutseedup = wx.CheckBox(p, -1, self.localize('timeoutup'))
        timeoutseedwatch.Add(self.timeoutseedup)
        self.timeoutseedtrack = wx.CheckBox(p, -1, self.localize('timeouttrack'))
        timeoutseedwatch.Add(self.timeoutseedtrack, 0, wx.TOP, 1)
        seedbox.Add(timeoutseedwatch, 0, wx.LEFT, 15)
        workseedbox.Add(seedbox, 1, wx.EXPAND | wx.LEFT, 3)

        timeoutsettingbox.Add(workseedbox, 0, wx.EXPAND | wx.TOP, 3)

        sizer.Add(timeoutsettingbox, 0, wx.EXPAND | wx.TOP, 3)

        timeoutvaluesbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('timeoutvalues')), wx.VERTICAL)
        if self.multitorrent:
            self.timeoutvaluesswitch = wx.CheckBox(p, -1, '')
            timeoutvaluesbox.Add(self.timeoutvaluesswitch, 0, wx.ALIGN_RIGHT | wx.TOP, -17)

        # Timeout for down stream (minutes)
        downupbox = wx.FlexGridSizer(cols = 2, hgap = 5)
        downupbox.AddGrowableCol(0)
        downupbox.AddGrowableCol(1)
        download_val = ['oo', '10', '20', '30', '60', '90', '120', '150', '180', '210', '240', '300', '360', '420', '480', '960']
        self.cb_download = wx.ComboBox(p, -1, '', wx.Point(-1, -1),
                                       wx.Size(-1, -1), download_val, wx.CB_DROPDOWN | wx.CB_READONLY)
        download_box = wx.BoxSizer(wx.HORIZONTAL)
        download_box.Add(wx.StaticText(p, -1, self.localize('timeout_download')), 0, wx.ALIGN_CENTER_VERTICAL)
        download_box.Add(self.cb_download, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        download_box.Add(wx.StaticText(p, -1, self.localize('minute_short')), 0, wx.ALIGN_CENTER_VERTICAL)
        downupbox.Add(download_box, 1)

        # Timeout for up stream (hours)
        upload_val = ['oo', '0.5', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '15', '20', '24', '36', '48', '120', '240']
        self.cb_upload = wx.ComboBox(p, -1, '', wx.Point(-1, -1),
                                     wx.Size(-1, -1), upload_val, wx.CB_DROPDOWN | wx.CB_READONLY)
        upload_box = wx.BoxSizer(wx.HORIZONTAL)
        upload_box.Add(wx.StaticText(p, -1, self.localize('timeout_upload')), 0, wx.ALIGN_CENTER_VERTICAL)
        upload_box.Add(self.cb_upload, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        upload_box.Add(wx.StaticText(p, -1, self.localize('hour_short')), 0, wx.ALIGN_CENTER_VERTICAL)
        downupbox.Add(upload_box, 1)

        timeoutvaluesbox.Add(downupbox, 1, wx.EXPAND | wx.TOP, 0)

        # Timeout for contacting tracker (minutes)
        tracker_val = ['oo', '5', '10', '15', '30', '45', '60', '120', '180', '240', '300', '360', '420', '480', '960']
        self.cb_tracker = wx.ComboBox(p, -1, '', wx.Point(-1, -1),
                                      wx.Size(-1, -1), tracker_val, wx.CB_DROPDOWN | wx.CB_READONLY)
        tracker_box = wx.BoxSizer(wx.HORIZONTAL)
        tracker_box.Add(wx.StaticText(p, -1, self.localize('timeout_tracker')), 0, wx.ALIGN_CENTER_VERTICAL)
        tracker_box.Add(self.cb_tracker, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        tracker_box.Add(wx.StaticText(p, -1, self.localize('minute_short')), 0, wx.ALIGN_CENTER_VERTICAL)

        timeoutvaluesbox.Add(tracker_box, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.TOP, 3)

        sizer.Add(timeoutvaluesbox, 0, wx.EXPAND | wx.TOP, 3)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)
        return p

    # Group Label/Move/Tracker
    ###########################
    def makeLabMovTraPanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        labelbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('labelsetting')), wx.VERTICAL)
        if self.multitorrent:
            self.labelswitch = wx.CheckBox(p, -1, '')
            labelbox.Add(self.labelswitch, 0, wx.ALIGN_RIGHT | wx.TOP, -17)

        shortlabelbox = wx.BoxSizer(wx.HORIZONTAL)
        shortlabelbox.Add(wx.StaticText(p, -1, self.localize('shortlabel')), 0, wx.ALIGN_CENTER_VERTICAL)
        self.shortlabel = wx.TextCtrl(p, -1, '', wx.Point(-1, -1), wx.Size(-1, -1))        
        shortlabelbox.Add(self.shortlabel, 1, wx.EXPAND | wx.LEFT, 5)

        labelbox.Add(shortlabelbox, 0, wx.EXPAND | wx.TOP, self.firstvertgap)

        movefolderbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('movingsetting')), wx.VERTICAL)
        if self.multitorrent:
            self.movefolderswitch = wx.CheckBox(p, -1, '')
            movefolderbox.Add(self.movefolderswitch, 0, wx.ALIGN_RIGHT | wx.TOP, -17)
        self.movefolder = wx.TextCtrl(p, -1, "", size = (-1, -1))
        movedirbrowsebtn = wx.Button(p, -1, "...", wx.Point(-1, -1), wx.Size(20, 20))
        self.Bind(wx.EVT_BUTTON, self.onMoveDirBrowseDir, movedirbrowsebtn)
        movedirbox = wx.BoxSizer(wx.HORIZONTAL)
        movedirbox.Add(wx.StaticText(p, -1, self.localize('setmovefolder')), 0, wx.ALIGN_CENTER_VERTICAL)
        movedirbox.Add(self.movefolder, 1, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        movedirbox.Add(movedirbrowsebtn, 0, wx.ALIGN_CENTER_VERTICAL)

        movefolderbox.Add(movedirbox, 0, wx.EXPAND | wx.TOP, self.firstvertgap)

        exttrackerbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('exttracker')), wx.VERTICAL)
        if self.multitorrent:
            self.exttrackerswitch = wx.CheckBox(p, -1, '')
            exttrackerbox.Add(self.exttrackerswitch, 0, wx.ALIGN_RIGHT | wx.TOP, -17)

        self.exttracker = wx.CheckBox(p, -1, self.localize('exttrackersw'))        
        exttrackerbox.Add(self.exttracker, 0, wx.TOP, self.firstvertgap)

        exttrackerurlbox = wx.BoxSizer(wx.HORIZONTAL)
        exttrackerurlbox.Add(wx.StaticText(p, -1, self.localize('exttrackerurl')), 0, wx.ALIGN_CENTER_VERTICAL)
        self.exttrackerurl = wx.TextCtrl(p, -1, '', wx.Point(-1, -1), wx.Size(-1, -1))        
        exttrackerurlbox.Add(self.exttrackerurl, 1, wx.EXPAND | wx.LEFT, 5)
        exttrackerbox.Add(exttrackerurlbox, 0, wx.EXPAND | wx.TOP, 3)

        sizer.Add(labelbox, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(movefolderbox, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(exttrackerbox, 0, wx.EXPAND | wx.TOP, 3)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)
        return p

    def onLeftNoteBook(self, event):
        event.Skip()
        if not self.multitorrent:
            return
        clicked, where = self.notebook.HitTest((event.m_x, event.m_y))
        if where == wx.NB_HITTEST_ONLABEL:
            if clicked == self.notebook.GetSelection():
                if clicked == 0:
                    if self.downloadswitch.GetValue() and self.uploadswitch.GetValue() and self.continueswitch.GetValue():
                        self.downloadswitch.SetValue(False)
                        self.uploadswitch.SetValue(False)
                        self.continueswitch.SetValue(False)
                    else:
                        self.downloadswitch.SetValue(True)
                        self.uploadswitch.SetValue(True)
                        self.continueswitch.SetValue(True)
                elif clicked == 1:
                    if self.timeoutsettingswitch.GetValue() and self.timeoutvaluesswitch.GetValue():
                        self.timeoutsettingswitch.SetValue(False)
                        self.timeoutvaluesswitch.SetValue(False)
                    else:
                        self.timeoutsettingswitch.SetValue(True)
                        self.timeoutvaluesswitch.SetValue(True)
                elif clicked == 2:
                    if self.labelswitch.GetValue() and self.movefolderswitch.GetValue() and self.exttrackerswitch.GetValue():
                        self.labelswitch.SetValue(False)
                        self.movefolderswitch.SetValue(False)
                        self.exttrackerswitch.SetValue(False)
                    else:
                        self.labelswitch.SetValue(True)
                        self.movefolderswitch.SetValue(True)
                        self.exttrackerswitch.SetValue(True)

    def onMouseWheel(self, event):
        nextpage = self.notebook.GetSelection()
        if event.m_wheelRotation > 0:
            if nextpage < self.maxpage:
                nextpage += 1
            else:
                nextpage = 0
            self.notebook.SetSelection(nextpage)
        else:
            if nextpage > 0:
                nextpage -= 1
            else:
                nextpage = self.maxpage
            self.notebook.SetSelection(nextpage)

    def onMaxUploadSpin(self, event):
        self.maxupload.SetValue(str(event.GetPosition()))

    def onMoveDirBrowseDir(self, event):
        foldername = wx.DirDialog(self, self.localize('choosemovefolder'), 
                                 style = wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if foldername.ShowModal() == wx.ID_OK:
            self.movefolder.SetValue(foldername.GetPath())
        foldername.Destroy()

    def onCancel(self, event = None):
        self.EndModal(wx.ID_CANCEL)

    def onApply(self, event):
        tor_info = []
        if not self.multitorrent or self.labelswitch.GetValue():
            tor_info.append(self.shortlabel.GetValue())
        else:
            tor_info.append(None)
        if not self.multitorrent or self.downloadswitch.GetValue() and self.downloadrateswitch.GetValue():
            # Check download rate is integer
            try:
                download_rate = int(self.downloadrate.GetValue())
            except:
                dlg = wx.MessageDialog(self, self.localize('downloadrateintwarning'),
                                       self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                return
            # Check if >= 0
            if download_rate < 0:
                dlg = wx.MessageDialog(self, self.localize('downloadrateminwarning'),
                                       self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                return
            tor_info.append(download_rate)
        else:
            tor_info.append(None)
        if not self.multitorrent or self.downloadswitch.GetValue() and self.prioritizedownswitch.GetValue():
            tor_info.append(int(self.prioritizedown.GetValue()))
        else:
            tor_info.append(None)
        if not self.multitorrent or self.uploadswitch.GetValue() and self.maxuploadswitch.GetValue():
            tor_info.append(self.maxupload.GetValue())
        else:
            tor_info.append(None)
        if not self.multitorrent or self.uploadswitch.GetValue() and self.uploadrateswitch.GetValue():
            # Check upload rate is integer
            try:
                upload_rate = int(self.uploadrate.GetValue())
            except:
                dlg = wx.MessageDialog(self, self.localize('uploadrateintwarning'),
                                       self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                return
            # Check if < minuprate and != 0
            if upload_rate < self.utility.minuprate and upload_rate != 0:
                dlg = wx.MessageDialog(self, self.localize('uploadrateminwarningauto') % self.utility.minuprate,
                                       self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                return
            tor_info.append(upload_rate)
        else:
            tor_info.append(None)
        if not self.multitorrent or self.uploadswitch.GetValue() and self.prioritizeupswitch.GetValue():
            tor_info.append(int(self.prioritizeup.GetValue()))
        else:
            tor_info.append(None)
        if not self.multitorrent or self.continueswitch.GetValue():
            for i in xrange(4):
                if self.rb[i].GetValue():
                    tor_info.append(str(i + 10 * int(self.uploadtillcopycomplete.GetValue())))
                    break
            htime = self.cbhtime.GetValue()
            if not htime:
                htime = '0'
            tor_info.append(htime)
            mtime = self.cbmtime.GetValue()
            if not mtime:
                mtime = '0'
            tor_info.append(mtime)
            tor_info.append(self.cbratio.GetValue())
        else:
            tor_info.extend(4 * [None])
        if not self.multitorrent or self.timeoutsettingswitch.GetValue():
            tor_info.append(int(self.timeoutswitch.GetValue()))
            tor_info.append(self.timeoutaction.GetSelection())
            tor_info.append(int(self.timeoutwork.GetValue()))
            tor_info.append(int(self.timeoutworkdown.GetValue()))
            if self.timeoutworktrack.IsEnabled():
                tor_info.append(int(self.timeoutworktrack.GetValue()))
            else:
                tor_info.append(None)
            tor_info.append(int(self.timeoutseed.GetValue()))
            tor_info.append(int(self.timeoutseedup.GetValue()))
            if self.timeoutseedtrack.IsEnabled():
                tor_info.append(int(self.timeoutseedtrack.GetValue()))
            else:
                tor_info.append(None)
        else:
            tor_info.extend(8 * [None])
        if not self.multitorrent or self.timeoutvaluesswitch.GetValue():
            tor_info.append(self.cb_tracker.GetValue())
            tor_info.append(self.cb_download.GetValue())
            tor_info.append(self.cb_upload.GetValue())
        else:
            tor_info.extend(3 * [None])
        if not self.multitorrent or self.movefolderswitch.GetValue():
            # Check if move folder is a valid Windows name
            tormovefolder = self.movefolder.GetValue()
            if sys.platform == 'win32' and tormovefolder and tormovefolder != self.torrent.movefolder:
                # We erase the final '\' except for a path like 'X:\'
                if tormovefolder and tormovefolder[-1] == '\\' and (len(tormovefolder) < 2 or tormovefolder[-2] != ':'):
                    tormovefolder = tormovefolder[:-1]
                tormovefolder = tormovefolder.rstrip(whitespace + '.')
                if not self.utility.checkWinPath(self, tormovefolder):
                    return
            tor_info.append(tormovefolder)
        else:
            tor_info.append(None)
        if not self.multitorrent or self.exttrackerswitch.GetValue():
            tor_info.append(int(self.exttracker.GetValue()))
            tor_info.append(self.exttrackerurl.GetValue())
        else:
            tor_info.extend(2 * [None])

        if self.panel.commandfromlist:
            self.queue.changeLocalInfo(self.panel.getSelected(), tor_info)
        else:
            try:
                index = self.queue.proctab.index(self.torrent)
            except:
                return True
            self.queue.changeLocalInfo([index], tor_info)

        return True

    def onOK(self, event):
        if self.onApply(event):
            self.EndModal(wx.ID_OK)
