##############################################################################
# Module : abcengine.py
# Author :
# Date   :
# Updated and modified for ABC_OKC : Old King Cole
#
# Description : Engine for a running torrent. Interfaces the output of a
#               BitTornado thread with ABC_OKC GUI.
#
##############################################################################
import sys, wx

from os import path, remove
from threading import Event, Semaphore, Thread, Timer
from time import clock, strftime
from traceback import print_exc
from StringIO import StringIO
from socket import error as socketerror
from hashlib import sha1

from BitTornado.download_bt1 import BT1Download, parse_params, get_response
from BitTornado.RawServer import RawServer
from BitTornado.natpunch import UPnP_test
from BitTornado.__init__ import createPeerID
from BitTornado.utility import exceptionArgsToString
from BitTornado.bencode import bdecode
from btmakemetafile import make_meta_data


FILESEM = Semaphore(1)

wxEVT_INVOKE = wx.NewEventType()


def EVT_INVOKE(win, func):
    win.Connect(-1, -1, wxEVT_INVOKE, func)

    
def DELEVT_INVOKE(win):
    win.Disconnect(-1, -1, wxEVT_INVOKE)


class InvokeEvent(wx.PyEvent):
    def __init__(self, func, args, kwargs, forcepost):
        wx.PyEvent.__init__(self)
        self.SetEventType(wxEVT_INVOKE)
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.forcepost = forcepost


class ABCEngine(wx.EvtHandler):
    def __init__(self, parent, guiman, torrent, listid, auto, onlycheck,
                 fastresume, reseed, btparams, sbwaitingforleechers, dht):
        wx.EvtHandler.__init__(self)
        self.dow = None
        self.parent = parent
        self.guiman = guiman
        self.listid = listid
        self.torrent = torrent
        self.btparams = btparams
        self.fastresume = fastresume
        self.reseed = reseed
        self.utility = self.parent.utility
        self.localize = self.utility.lang.get
        self.abcparams = self.utility.abcparams
        self.getRankfromID = self.guiman.getRankfromID
        self.list = parent.list
        self.doneflag = Event()
        self.fin = False
        self.stoppedonrequest = False
        # Seeding and total seeding pseudo timers
        # 0 : stopped
        # 1 : running
        self.seedingtimer = 0
        self.totalseedingtimer = 0
        # Last seeding times before current seeding
        self.lastseedingtime = torrent.seedingtime
        self.lasttotalseedingtime = torrent.totalseedingtime
        self.workingtimeleft = 0
        self.detailwin = torrent.detailwin
        self.infowin = torrent.infowin
        self.superseed = False
        self.reannouncelast = 0
        self.olddownsize = torrent.downsize
        self.oldupsize = torrent.upsize
        self.newdownsize = 0.
        self.newupsize = 0.
        # Down and up size from previous refresh
        self.prevdownsize = 0.
        self.prevupsize = 0.
        self.shareratio = 0.
        self.forceseeding = torrent.uploadtillcopycomplete
        self.normalseedingover = False
        self.numpeer = '0'
        self.numseed = '0'
        self.numpeertext = '0 (' + torrent.peer + ')'
        self.numseedtext = '0 (' + torrent.seed + ')'
        self.ispaused = False
        self.guilastupdate = self.spewwait = self.lasttorrentactive = clock()
        self.firsterrortracker = 0

        self.btstatus = "waiting"
        self.awaken = False
        if onlycheck:
            self.progress = 0.
        else:
            self.progress = torrent.progress
        self.downrate = 0.
        self.uprate = 0.
        self.pexuprate = 0.
        self.shareratiotext = "0.0%"
        self.numconnections = 0
        self.numdownloads = 0
        self.numuploads = 0
        self.numdeaddown = 0
        self.peeravgtext = "0.0%"
        self.totalspeed = 0.
        self.downloadrate = self.utility.formatedRate(0, self.torrent.prioritizedown)
        self.uploadrate = self.utility.formatedRate(0, self.torrent.prioritizeup)
        self.totalspeedtext = self.utility.formatedRate(0)
        self.dlsizetext = self.utility.formatedSize(self.olddownsize)
        self.ulsizetext = self.utility.formatedSize(self.oldupsize)
        self.noupdateGUI = False
        self.workingtimelastcheck = 0.
        self.seedingtimelastcheck = 0.
        self.seedingtimerreset = 0.
        self.totalseedingtimerreset = 0.
        self.statistics = None
        self.spew = None
        # To compute mean over 10 last values of uprate and downrate
        self.pastuprate = 10 * [0.]
        self.pastdownrate = 10 * [0.]
        self.meanuprate = 0
        self.meandownrate = 0
        self.stopafterfirstrefresh = False
        self.btcomplete = False
        self.dht = dht
        self.filespriochangeallowed = False

        # Keep track of if we have any connections
        self.hasconnections = False

        # Automatically or manually started
        self.auto = auto

        self.colour = self.utility.colnoeng

        # Counters for activity
        self.timeractivityoff = self.timeractivityon = 0

        # True if only check the torrent
        self.onlycheck = onlycheck

        # True if the torrent is already complete when it is started
        self.alreadycomplete = torrent.complete

        # True if timeouts are currently canceled
        self.timeoutscanceled = False

        # To keep track of change in uploadtimemax
        self.olduploadtimemax = torrent.uploadtimemax

        # True once magnet has got its metadata and starts updating its storage
        self.restarted = False

        # Last time a standby uncomplete torrent with leechers was resumed
        # to check the number of available copies and leechers progress
        # Before being initiated with this delay only when a torrent possibly ends checking its data,
        # it's True or False to say that this mode must be engaged.
        # False means the torrent is not waiting for connections from leechers and
        # must not be checked for the number of available copies and leechers progress
        self.sbwaitingforleechers = sbwaitingforleechers

        # Flag set when BitTornado thread is finished
        self.shutdownflag = Event()

        EVT_INVOKE(self, self.onInvoke)

    def onInvoke(self, event):
        if event.forcepost or not self.doneflag.isSet():
            event.func(*event.args, **event.kwargs)

    def invokeLater(self, func, args = [], kwargs = {}, forcepost = False):
        if forcepost or not self.doneflag.isSet():
            wx.PostEvent(self, InvokeEvent(func, args, kwargs, forcepost))

    def formatedProgress(self):
        if self.progress == -1:
            return '  ?  %' + self.torrent.partialMarker()
        progressstring = str(.1 * int(10 * self.progress))
        if progressstring == '100.0':
            if self.torrent.isCheckingOrAllocating() or self.torrent.complete:
                return '100%' + self.torrent.partialMarker()
            return '99.9%' + self.torrent.partialMarker()
        return progressstring + '%' + self.torrent.partialMarker()

    def updateDetailWindowNow(self, detwin):
        self.detailwin = detwin
        self.spewwait = 0
        self.updateDetailWindow()

    def updateStatus(self, dpflag = Event(), fractionDone = None, downRate = None, upRate = None, pexUpRate = None,
                     activity = None, statistics = None, spew = None, sizeDone = None):
        self.invokeLater(self.onUpdateStatus, [dpflag, fractionDone, downRate, upRate, pexUpRate, activity, statistics,
                                               spew, sizeDone])

    def onUpdateStatus(self, dpflag, fractionDone, downRate, upRate, pexUpRate, activity, statistics, spew, sizeDone):
        self.statistics = statistics
        self.spew = spew
        if activity is not None and not self.fin:
            self.btstatus = activity

        if self.noupdateGUI or self.guilastupdate + self.utility.GUIupdaterate > clock():
            dpflag.set()
            return

        self.guilastupdate = clock()

        # Check timeouts
        if self.timeoutscanceled:
            self.lasttorrentactive = self.guilastupdate
        else:
            # Check timeout when seeding
            if self.fin:
                if upRate is not None and upRate == 0 or self.statistics and self.statistics.numPeers == 0:
                    if self.torrent.timeoutswitch and self.torrent.timeoutseed and self.torrent.timeoutseedup and self.torrent.timeoutupload != 'oo' \
                       and self.guilastupdate - self.lasttorrentactive > float(self.torrent.timeoutupload) * 3600 \
                       and self.parent.isTimeoutAllowed():
                        self.lasttorrentactive = self.guilastupdate
                        if self.onTimeout(self.localize('timeoutupmsg')):
                            return
                else:
                    self.lasttorrentactive = self.guilastupdate
    
            # Check timeout when working
            else:
                if downRate is not None and downRate == 0 or self.statistics and self.statistics.numPeers + self.statistics.numSeeds + self.statistics.numOldSeeds == 0:
                    if self.torrent.timeoutswitch and self.torrent.timeoutwork and self.torrent.timeoutworkdown and self.torrent.timeoutdownload != 'oo' \
                       and self.guilastupdate - self.lasttorrentactive > float(self.torrent.timeoutdownload) * 60 \
                       and self.parent.isTimeoutAllowed():
                        self.lasttorrentactive = self.guilastupdate
                        if self.onTimeout(self.localize('timeoutdownmsg')):
                            return
                else:
                    self.lasttorrentactive = self.guilastupdate

        # Set colour
        oldcolour = self.colour
        if self.statistics is None:
            # Start up
            self.colour = self.utility.colnoeng
            self.hasconnections = False
        elif self.statistics.numPeers + self.statistics.numSeeds + self.statistics.numOldSeeds == 0:
            if self.statistics.last_failed:
                # Disconnected
                self.colour = self.utility.coldisc
            else:
                # No connections
                self.colour = self.utility.colnocon
            self.hasconnections = False
        elif not self.statistics.external_connection_made:
            # No incoming
            self.colour = self.utility.colnoinc
            self.hasconnections = True
        elif (self.statistics.numSeeds + self.statistics.numOldSeeds == 0) \
             and ((self.fin and self.statistics.numCopies < 1) \
             or (not self.fin and self.statistics.numCopies2 < 1)):
            # No completes
            self.colour = self.utility.colnocom
            self.hasconnections = True
        else:
            # All good
            self.colour = self.utility.colok
            self.hasconnections = True

        if not self.fin:
            if self.torrent.magnet:
                self.progress = 0.0
            elif fractionDone is not None:
                self.progress = fractionDone * 100
            if activity is None and self.btstatus != 'waiting':
                if self.hasconnections:
                    self.btstatus = 'working'
                else:
                    self.btstatus = 'connectingtopeers'

        if self.fin:
            self.downrate = 0.
            self.downloadrate = self.utility.formatedRate(None, self.torrent.prioritizedown)
        elif downRate is None:
            self.downrate = 0.
            if self.restarted:
                self.downloadrate = self.utility.formatedRate(0, self.torrent.prioritizedown)
            else:
                self.downloadrate = self.utility.formatedRate(None, self.torrent.prioritizedown)
        else:
            self.downrate = float(downRate) / 1024
            self.downloadrate = self.utility.formatedRate(self.downrate, self.torrent.prioritizedown)

        if upRate is None:
            self.uprate = 0.
            if self.restarted:
                self.uploadrate = self.utility.formatedRate(0, self.torrent.prioritizeup)
            else:
                self.uploadrate = self.utility.formatedRate(None, self.torrent.prioritizeup)
        else:
            self.uprate = float(upRate) / 1024
            self.uploadrate = self.utility.formatedRate(self.uprate, self.torrent.prioritizeup)

        if pexUpRate is not None:
            self.pexuprate = float(pexUpRate) / 1024
        else:
            self.pexuprate = 0.

        # Compute mean uprate and downrate
        self.meanuprate += (self.uprate - self.pastuprate[0]) / 10.
        self.pastuprate.pop(0)
        self.pastuprate.append(self.uprate)
        self.meandownrate += (self.downrate - self.pastdownrate[0]) / 10.
        self.pastdownrate.pop(0)
        self.pastdownrate.append(self.downrate)

        if self.statistics is None:
            self.numpeer = "0"
            self.numseed = "0"
        else:
            self.numpeer = str(self.statistics.numPeers)
            self.peeravgtext = str(.1 * int(10 * self.statistics.percentDone)) + '%'
            self.dlsizetext = self.utility.formatedSize(self.olddownsize + self.statistics.downTotal)
            self.ulsizetext = self.utility.formatedSize(self.oldupsize + self.statistics.upTotal)
            self.totalspeed = self.statistics.torrentRate / 1024.
            self.totalspeedtext = self.utility.formatedRate(self.totalspeed)
            self.numconnections = self.statistics.numPeers
            if self.fin:
                self.numseed = str(self.statistics.numOldSeeds)
                self.torrent.numcopy = '%.3f' % (.001 * int(1000 * self.statistics.numCopies))
            else:
                self.numseed = str(self.statistics.numSeeds)
                self.numconnections += self.statistics.numSeeds
                self.torrent.numcopy = '%.3f' % (.001 * int(1000 * self.statistics.numCopies2))
            self.newdownsize = float(self.statistics.downTotal)
            self.parent.downvolmeasure.update_rate(self.newdownsize - self.prevdownsize)
            if self.parent.downvolcheckcycle == 0:
                self.parent.downvolfixedwin += self.newdownsize - self.prevdownsize
            self.prevdownsize = self.newdownsize
            self.newupsize = float(self.statistics.upTotal)
            self.parent.upvolmeasure.update_rate(self.newupsize - self.prevupsize)
            if self.parent.upvolcheckcycle == 0:
                self.parent.upvolfixedwin += self.newupsize - self.prevupsize
            self.prevupsize = self.newupsize
            if self.olddownsize + self.newdownsize > 0:
                self.shareratio = (self.oldupsize + self.newupsize) / (self.olddownsize + self.newdownsize) * 100
            else:
                if self.torrent.requestedsize <= 0 and not self.torrent.magnet:
                    self.shareratio = 999.9
                elif self.progress <= 0:
                    self.shareratio = 0.
                else:
                    self.shareratio = (self.oldupsize + self.newupsize) / self.torrent.requestedsize / self.progress * 10000
            self.shareratiotext = str(.1 * int(10 * self.shareratio)) + "%"
        self.numpeertext = self.numpeer + ' (' + self.torrent.peer + ')'
        self.numseedtext = self.numseed + ' (' + self.torrent.seed + ')'

        if self.colour != oldcolour:
            self.updateColour()

        if self.fin:
            self.countSeedingTimes(self.guilastupdate)
        else:
            self.countWorkingTimeLeft(self.guilastupdate, sizeDone)

        self.refreshDisplay()

        self.updateInfoWindow()
        self.updateDetailWindow()

        if self.stopafterfirstrefresh:
            self.parent.procSTOP([self.listid])
            return

        dpflag.set()

        # Check if torrent is completed
        if not self.fin and self.torrent.unwantedfiles and not self.torrent.isCheckingOrAllocating():
            self.reportProgress()
            self.torrent.checkComplete()
            if self.btcomplete:
                self.completed(False)

        # Check for #seeds and #copies to set forceseeding
        if self.fin:
            # Check if must terminate
            if (self.torrent.uploadoption == "1" or self.torrent.uploadoption == "3") and self.torrent.seedingtimeleft == 0 \
               or (self.torrent.uploadoption == "2" or self.torrent.uploadoption == "3") and self.torrent.ratioIsReached():
                if self.normalseedingover:
                    if not self.forceseeding:
                        self.terminateUpload()
                else:
                    # First time normal seeding is over
                    self.checkForTerminating()
            else:
                self.normalseedingover = False

            if self.btstatus != 'finished' and self.torrent.uploadtillcopycomplete:
                if self.statistics is None or (self.statistics.numSeeds + self.statistics.numOldSeeds == 0 and self.statistics.numCopies < 1):
                    if not self.torrent.scraping and self.torrent.scrapeisfresh \
                       and self.torrent.seed != self.utility.notrackeranswermarker \
                       and self.torrent.seed != self.utility.notrackermarker \
                       and int(self.torrent.seed) > 1:
                        self.forceseeding = False
                    else:
                        self.forceseeding = True
                else:
                    self.forceseeding = False

        # Get scrape data every scrapeauto minutes
        if self.btstatus != 'finished' \
           and (self.abcparams['scrape'] == "1" or (self.fin and self.normalseedingover and self.forceseeding)):
            if self.statistics is not None and not self.torrent.scraping and (self.torrent.lastgetscrape == 0
               or self.guilastupdate - self.torrent.lastgetscrape >= self.utility.scrapeauto):
                self.parent.getScrapeData(self.torrent, autoscraping = True)

    def refreshIfNeeded(self, rank, field):
        if self.list.GetItem(self.listid, rank).GetText() != field:
            self.list.SetStringItem(self.listid, rank, field)

    def refreshDisplay(self):
        # Progress
        rank = self.getRankfromID(5)
        if rank != -1:
            self.refreshIfNeeded(rank, self.formatedProgress())
        # Status
        rank = self.getRankfromID(6)
        if rank != -1:
            self.refreshIfNeeded(rank, self.localize(self.btstatus))
        # Time remaining
        rank = self.getRankfromID(8)
        if rank != -1:
            if self.fin:
                if self.torrent.uploadoption != "0" and not (self.forceseeding and self.normalseedingover):
                    self.refreshIfNeeded(rank, self.localize('etaS') + " " + self.utility.eta_value(self.torrent.seedingtimeleft))
            elif self.torrent.complete:
                self.refreshIfNeeded(rank, self.localize('etaS') + "      ?")
            elif self.torrent.isWaiting() or self.torrent.isCheckingOrAllocating():
                if self.alreadycomplete:
                    self.refreshIfNeeded(rank, self.localize('etaS') + "      ?")
                else:
                    self.refreshIfNeeded(rank, self.localize('etaD') + "      ?")
            else:
                self.refreshIfNeeded(rank, self.localize('etaD') + " " + self.utility.eta_value(self.workingtimeleft))
        # DL speed
        rank = self.getRankfromID(10)
        if rank != -1:
            self.refreshIfNeeded(rank, self.downloadrate[0])
        # UL speed
        rank = self.getRankfromID(11)
        if rank != -1:
            self.refreshIfNeeded(rank, self.uploadrate[0])
        if self.statistics is not None:
            # Share ratio
            rank = self.getRankfromID(12)
            if rank != -1:
                self.refreshIfNeeded(rank, self.shareratiotext)
            # # seeds
            rank = self.getRankfromID(14)
            if rank != -1:
                self.refreshIfNeeded(rank, self.numseedtext)
            # # peers
            rank = self.getRankfromID(15)
            if rank != -1:
                self.refreshIfNeeded(rank, self.numpeertext)
            # # copies
            rank = self.getRankfromID(16)
            if rank != -1:
                self.refreshIfNeeded(rank, self.torrent.numcopy)
            # Peer avg progress
            rank = self.getRankfromID(17)
            if rank != -1:
                self.refreshIfNeeded(rank, self.peeravgtext)
            # Download size
            rank = self.getRankfromID(18)
            if rank != -1:
                self.refreshIfNeeded(rank, self.dlsizetext[0])
            # Upload size
            rank = self.getRankfromID(19)
            if rank != -1:
                self.refreshIfNeeded(rank, self.ulsizetext[0])
            # Total speed
            rank = self.getRankfromID(20)
            if rank != -1:
                self.refreshIfNeeded(rank, self.totalspeedtext[0])
        if self.fin:
            # Seeding time
            rank = self.getRankfromID(27)
            if rank != -1:
                self.refreshIfNeeded(rank, self.utility.eta_value(self.torrent.seedingtime, maxvalue = False))
            # Total seeding time
            rank = self.getRankfromID(28)
            if rank != -1:
                self.refreshIfNeeded(rank, self.utility.eta_value(self.torrent.totalseedingtime, maxvalue = False))

    def updateDetailWindow(self):
        if self.detailwin is None:
            # Only compute connections activity
            numdownloads = numuploads = numdeaddown = 0
            if self.spew is not None:
                for peer in self.spew:
                    if peer['uprate'] > 100:
                        numuploads += 1
                    if peer['downrate'] > 100:
                        numdownloads += 1
                    elif not self.torrent.complete and peer['dinterested'] \
                       and not peer['dchoked']:
                        numdeaddown += 1
            self.numdownloads = numdownloads
            self.numuploads = numuploads
            self.numdeaddown = numdeaddown
            return

        self.detailwin.totalprogress.SetLabel(self.formatedProgress())
        if self.torrent.status == 'pause' or self.torrent.status == 'onhold':
            self.detailwin.status.SetLabel(self.localize(self.torrent.status))
            self.detailwin.colour.SetBackgroundColour(self.utility.colnoeng)
        else:
            self.detailwin.status.SetLabel(self.localize(self.btstatus))
            self.detailwin.colour.SetBackgroundColour(self.colour)
        self.detailwin.colour.Refresh()

        self.detailwin.downsize.SetLabel(self.utility.sizeWithUnit(self.dlsizetext))
        self.detailwin.upsize.SetLabel(self.utility.sizeWithUnit(self.ulsizetext))
        if not self.fin:
            self.detailwin.seedtitle.SetLabel(self.localize('dnumconnectedseed'))
        else:
            self.detailwin.seedtitle.SetLabel(self.localize('dseenseed'))
        self.detailwin.numseed.SetLabel(self.numseedtext)
        self.detailwin.numpeer.SetLabel(self.numpeertext)
        self.detailwin.numcopy.SetLabel(self.torrent.numcopy)
        self.detailwin.totalspeed.SetLabel(self.utility.rateWithUnit(self.totalspeedtext))
        if self.torrent.status == 'pause' or self.torrent.status == 'onhold':
            self.detailwin.downrate.SetLabel(self.utility.rateWithUnit('', self.torrent.prioritizedown))
            self.detailwin.uprate.SetLabel(self.utility.rateWithUnit('', self.torrent.prioritizeup))
        else:
            self.detailwin.downrate.SetLabel(self.utility.rateWithUnit(self.downloadrate, self.torrent.prioritizedown))
            self.detailwin.uprate.SetLabel(self.utility.rateWithUnit(self.uploadrate, self.torrent.prioritizeup))
        self.detailwin.avgprogress.SetLabel(self.peeravgtext)

        if self.spew is None:
            self.numdownloads = 0
            self.numuploads = 0
            self.numdeaddown = 0
        elif clock() - self.spewwait > 1:
            self.updateSpewList()

        if self.statistics is not None:
            if self.torrent.magnet:
                self.detailwin.storagestats1.SetLabel("     " + self.localize('downloadingmetadata'))
                self.detailwin.storagestats2.SetLabel("     ")
            else:
                self.detailwin.storagestats1.SetLabel("     " + self.localize('detailline1')
                                                      % (self.statistics.storage_active, self.statistics.storage_new,
                                                       self.statistics.storage_dirty))
                self.detailwin.storagestats2.SetLabel("     " + self.localize('detailline2')
                                                      % (self.statistics.storage_numcomplete, self.statistics.storage_totalpieces,
                                                      self.statistics.storage_justdownloaded, self.statistics.storage_numflunked))

    def updateSpewList(self):
        self.spewwait = clock()
        spewlist = self.detailwin.spewlist
        spewlen = len(self.spew) + 2

        if self.statistics is not None:
            kickbanlen = len(self.statistics.peers_kicked) + len(self.statistics.peers_banned)
            if kickbanlen:
                spewlen += kickbanlen + 1
        else:
            kickbanlen = 0

        for x in xrange(spewlen - spewlist.GetItemCount()):
            i = wx.ListItem()
            spewlist.InsertItem(i)
        for x in xrange(spewlen, spewlist.GetItemCount()):
            spewlist.DeleteItem(len(self.spew) + 1)

        tot_uprate = tot_downrate = numdownloads = numuploads = numdeaddown = row = 0
        for peer in self.spew:
            spewlist.SetStringItem(row, 0, self.utility.starflag[peer['optimistic']])
            spewlist.SetStringItem(row, 1, peer['ip'])
            spewlist.SetStringItem(row, 2, peer['direction'])

            uprate = peer['uprate']
            if uprate > 100:
                spewlist.SetStringItem(row, 3, '%.0f ' % (uprate / 1024) + self.utility.kbps)
                numuploads += 1
            else:
                spewlist.SetStringItem(row, 3, ' ')
            tot_uprate += uprate

            spewlist.SetStringItem(row, 4, self.utility.starflag[peer['uinterested']])
            spewlist.SetStringItem(row, 5, self.utility.starflag[peer['uchoked']])

            downrate = peer['downrate']
            if downrate > 100:
                spewlist.SetStringItem(row, 6, ('%.0f ' % (downrate / 1024)) + self.utility.kbps)
                numdownloads += 1
            else:
                spewlist.SetStringItem(row, 6, ' ')
                if not self.torrent.complete and peer['dinterested'] and not peer['dchoked']:
                    numdeaddown += 1
            tot_downrate += downrate

            spewlist.SetStringItem(row, 7, self.utility.starflag[peer['dinterested']])
            spewlist.SetStringItem(row, 8, self.utility.starflag[peer['dchoked']])
            spewlist.SetStringItem(row, 9, self.utility.starflag[peer['snubbed']])

            if peer['utotal'] is None:
                spewlist.SetStringItem(row, 10, '')
            else:
                spewlist.SetStringItem(row, 10, '%.2f ' % (float(peer['utotal']) / 1048576) + self.utility.mb)

            spewlist.SetStringItem(row, 11, ('%.2f ' % (float(peer['dtotal']) / 1048576)) + self.utility.mb)

            if peer['completed'] is None:
                spewlist.SetStringItem(row, 12, '?')
            else:
                spewlist.SetStringItem(row, 12, str(.1 * int(1000 * peer['completed'])) + '%')

            if peer['speed'] is None:
                spewlist.SetStringItem(row, 13, '?')
            else:
                spewlist.SetStringItem(row, 13, ('%.0f ' % (float(peer['speed']) / 1024)) + self.utility.kbps)

            row += 1

        self.numdownloads = numdownloads
        self.numuploads = numuploads
        self.numdeaddown = numdeaddown


        for i in xrange(14):
            spewlist.SetStringItem(row, i, '')

        row += 1
        spewlist.SetStringItem(row, 1, '         ' + self.localize('TOTALS'))
  
        spewlist.SetStringItem(row, 3, ('%.0f ' % (float(tot_uprate) / 1024)) + self.utility.kbps)
        spewlist.SetStringItem(row, 6, ('%.0f ' % (float(tot_downrate) / 1024)) + self.utility.kbps)

        if self.statistics is not None:
            spewlist.SetStringItem(row, 10, ('%.2f ' % (float(self.statistics.upTotal) / 1048576)) + self.utility.mb)
            spewlist.SetStringItem(row, 11, ('%.2f ' % (float(self.statistics.downTotal) / 1048576)) + self.utility.mb)
        else:
            spewlist.SetStringItem(row, 10, '')
            spewlist.SetStringItem(row, 11, '')

        for i in [0, 2, 4, 5, 7, 8, 9, 12, 13]:
            spewlist.SetStringItem(row, i, '')

        if kickbanlen:
            row += 1
            for i in xrange(14):
                spewlist.SetStringItem(row, i, '')

            for ip in self.statistics.peers_kicked:
                row += 1
                spewlist.SetStringItem(row, 1, ip[0])
                spewlist.SetStringItem(row, 3, self.localize('KICKED'))
                for i in [0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13]:
                    spewlist.SetStringItem(row, i, '')

            for ip in self.statistics.peers_banned:
                row += 1
                spewlist.SetStringItem(row, 1, ip[0])
                spewlist.SetStringItem(row, 3, self.localize('BANNED'))
                for i in [0, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13]:
                    spewlist.SetStringItem(row, i, '')

    def reportProgress(self, statistics = None):
        if statistics is None:
            statistics = self.statistics
        if statistics is not None and not statistics.singlefile:
            if self.torrent.filesprogress is None:
                if self.torrent.complete and not self.torrent.unwantedfiles:
                    self.torrent.filesprogress = self.torrent.filesnumber * ['Done']
                else:
                    self.torrent.filesprogress = self.torrent.filesnumber * ['0']
            for i in xrange(self.torrent.filesnumber):
                if self.torrent.filespriority is None or self.torrent.filespriority[i] >= 0:
                    if statistics.fileinplace[i]:
                        self.torrent.filesprogress[i] = 'Done'
                    elif statistics.filecomplete[i]:
                        self.torrent.filesprogress[i] = '100'
                    else:
                        frac = statistics.fileamtdone[i] * 100
                        if frac < 0:
                            frac = 0.
                        self.torrent.filesprogress[i] = str(frac)
                else:
                    self.torrent.filesprogress[i] = '0'
        else:
            self.torrent.filesprogress = None

    def updateInfoWindow(self):
        if self.infowin is None: 
            return

        self.infowin.totalprogress.SetLabel(self.formatedProgress())
        self.infowin.status.SetLabel(self.localize(self.btstatus))
        self.infowin.colour.SetBackgroundColour(self.colour)
        self.infowin.colour.Refresh()

        self.infowin.downrate.SetLabel(self.utility.rateWithUnit(self.downloadrate, self.torrent.prioritizedown))
        self.infowin.uprate.SetLabel(self.utility.rateWithUnit(self.uploadrate, self.torrent.prioritizeup))

        if not self.torrent.singlefile and self.statistics is not None \
           and not self.statistics.singlefile and self.statistics.filelistupdated.isSet():
            progresscol = 3 + self.infowin.originalfilenamesshown
            for i in xrange(self.torrent.filesnumber):
                if self.torrent.filespriority is None or self.torrent.filespriority[i] >= 0:
                    if self.statistics.fileinplace[i]:
                        self.infowin.filelist.SetStringItem(i, progresscol, self.localize('done'))
                    elif self.statistics.filecomplete[i]:
                        self.infowin.filelist.SetStringItem(i, progresscol, "100%")
                    else:
                        frac = self.statistics.fileamtdone[i] * 100
                        if frac < 0:
                            frac = 0.
                        self.infowin.filelist.SetStringItem(i, progresscol, '%d%%' % frac)
                else:
                    self.infowin.filelist.SetStringItem(i, progresscol, "0%")
            self.statistics.filelistupdated.clear()

    def getDownUpSize(self):
        return (self.olddownsize + self.newdownsize), (self.oldupsize + self.newupsize)

    def getInfo(self, fieldidlist):
        retmsg = ""
        for field in fieldidlist:
            if field == 4:
                retmsg += self.torrent.name + "|"
            elif field == 5:
                retmsg += self.formatedProgress() + "|"
            elif field == 6:
                if self.torrent.ismovingdata:
                    retmsg += self.localize('moving') + "|"
                else:
                    retmsg += self.localize(self.btstatus) + "|"
            elif field == 7:
                retmsg += self.utility.priorities_s[self.torrent.priority] + "|"
            elif field == 8:
                if self.fin:
                    if self.torrent.uploadoption == "0":
                        retmsg += self.localize('etaS') + '     oo|'
                    elif self.forceseeding and self.normalseedingover:
                        retmsg += self.localize('etaS') + " " + self.localize('tillcc') + "|"
                    else:
                        retmsg += self.localize('etaS') + " " + self.utility.eta_value(self.torrent.seedingtimeleft) + "|"
                elif self.torrent.complete:
                    retmsg += self.localize('etaS') + "      ?|"
                elif self.torrent.isWaiting() or self.torrent.isCheckingOrAllocating():
                    if self.alreadycomplete:
                        retmsg += self.localize('etaS') + "      ?|"
                    else:
                        retmsg += self.localize('etaD') + "      ?|"
                else:
                    retmsg += self.localize('etaD') + " " + self.utility.eta_value(self.workingtimeleft) + "|"
            elif field == 9:
                if self.torrent.magnet:
                    retmsg += "?|"
                else:
                    retmsg += self.utility.sizeWithUnit(self.utility.formatedSize(self.torrent.requestedsize)) + "|"
            elif field == 10:
                retmsg += self.utility.formatedRate(self.downrate)[0] + "|"
            elif field == 11:
                retmsg += self.utility.formatedRate(self.uprate)[0] + "|"
            elif field == 12:
                retmsg += self.shareratiotext + "|"
            elif field == 13:
                retmsg += self.torrent.message + "|"
            elif field == 14:
                retmsg += self.numseedtext + "|"
            elif field == 15:
                retmsg += self.numpeertext + "|"
            elif field == 16:
                retmsg += self.torrent.numcopy + "|"
            elif field == 17:
                retmsg += self.peeravgtext + "|"
            elif field == 18:
                retmsg += self.utility.sizeWithUnit(self.dlsizetext) + "|"
            elif field == 19:
                retmsg += self.utility.sizeWithUnit(self.ulsizetext) + "|"
            elif field == 20:
                retmsg += self.totalspeedtext[0] + "|"
            elif field == 21:
                retmsg += path.split(self.torrent.src)[1] + "|"
            elif field == 22:
                if self.torrent.uploadoption == "0":
                    retmsg += "oo|"
                elif self.torrent.uploadoption == "1":
                    retmsg += self.utility.eta_value(self.torrent.uploadtimemax, seconds = False, maxvalue = False) + "|"
                elif self.torrent.uploadoption == "2":
                    retmsg += self.torrent.uploadratio + "%|"
                else:
                    retmsg += self.utility.eta_value(self.torrent.uploadtimemax, seconds = False, maxvalue = False) + \
                              self.torrent.uploadratio + "%|"
            elif field == 23:
                retmsg += self.torrent.shortlabel + "|"
            elif field == 24:
                retmsg += self.torrent.label + "|"
            elif field == 25:
                retmsg += self.torrent.activityMarker() + "|"
            elif field == 26:
                if torrent.magnet == 1:
                    retmsg += self.localize('notorrentinfo') + "|"
                else:
                    retmsg += self.torrent.dest + "|"
            elif field == 27:
                retmsg += self.utility.eta_value(self.torrent.seedingtime, maxvalue = False) + "|"
            elif field == 28:
                retmsg += self.utility.eta_value(self.torrent.totalseedingtime, maxvalue = False) + "|"
        return retmsg

    def completed(self, initphase):
        self.lasttorrentactive = clock()
        self.btcomplete = True
        if self.onlycheck:
            self.noupdateGUI = True
        self.invokeLater(self.onCompletedEvent, [initphase])

    def failed(self):
        if self.dow and self.dow.storage:
            for handle in self.dow.storage.handles.values():
                try:
                    handle.close()
                except:
                    continue
        self.invokeLater(self.onFailEvent, forcepost = True)

    def handleLimitedTorrent(self):
        self.stoppedonrequest = True
        self.invokeLater(self.onLimitedTorrent)

    def error(self, errormsg, rejectedonfirstreqcycle = False, external = False):
        self.invokeLater(self.onError, [errormsg, rejectedonfirstreqcycle, external], forcepost = True)

    def trackerOK(self, tracker, external = False):
        self.invokeLater(self.onTrackerOK, [tracker, external])

    def isTrackerCnxError(self, msg):
        for m in self.utility.trackercnxerrors:
            if msg.startswith(m):
                return True
        return False

    def onError(self, errormsg, rejectedonfirstreqcycle, external):
        now = clock()
        # Display error message
        self.torrent.displayMessage(strftime(self.utility.timestamp) + self.utility.messageheader + errormsg, self.listid)

        # If DHT is on and the torrent is not private, all tracker timeout features are disabled
        trackertimeoutcanceled = (self.dht and not self.torrent.private)

        # If an automatically started torrent gets rejected by all trackers on first request cycle and option set, stop it
        # just after the next update (to get the global progress and the progress for each file for a checking torrent)
        if not trackertimeoutcanceled and self.auto and self.abcparams['stoprejected'] == "1" and rejectedonfirstreqcycle:
            self.stopafterfirstrefresh = True

        # Check if all timeouts must be canceled because the tracker message contains
        # one of the strings specified in Preferences/Queue
        if self.abcparams['canceltimeouts'] == "1":
            for s in self.parent.canceltimeoutsmsg:
                if errormsg.find(s) >= 0:
                    self.firsterrortracker = 0
                    self.lasttorrentactive = now
                    self.timeoutscanceled = True
                    break
            else:
                self.timeoutscanceled = False

        # If failed connecting tracker in abcparams['timeouttracker'] mins
        # reduce its priority and force to queue
        if self.isTrackerCnxError(errormsg):
            if self.torrent.multitracker and not external:
                self.torrent.setDefaultTracker('')
            if not self.timeoutscanceled and not trackertimeoutcanceled:
                if self.firsterrortracker == 0:
                    self.firsterrortracker = now
                elif self.uprate == 0 and self.downrate == 0 \
                     and self.torrent.timeoutswitch and self.torrent.timeouttracker != 'oo' \
                     and (self.fin and self.torrent.timeoutseed and self.torrent.timeoutseedtrack \
                     or not self.fin and self.torrent.timeoutwork and self.torrent.timeoutworktrack) \
                     and now - self.firsterrortracker > int(self.torrent.timeouttracker) * 60 \
                     and self.parent.isTimeoutAllowed():
                    self.firsterrortracker = 0
                    self.onTimeout(self.localize('timeouttrackermsg'))

    def onTimeout(self, msg):
        if self.abcparams['displaytimeout'] == '1':
            self.torrent.displayMessage(strftime(self.utility.timestamp) + self.utility.messageheader + msg, self.listid)
        if self.torrent.timeoutaction == 1 and self.torrent.priority == 4 or self.torrent.timeoutaction == 2:
            self.parent.procSTOP([self.listid])
            return True
        if self.torrent.timeoutaction == 0 and self.abcparams['sbqueued'] == '0':
            self.parent.procSTANDBY([self.listid])
            return True
        willberesumed = self.parent.isNextToBeResumed(self.listid, self.torrent.priority + 1)
        if self.torrent.timeoutaction == 0:
            if willberesumed:
                self.torrent.incDecPriority(-1, self.listid)
                return False
            self.parent.procSTANDBY([self.listid])
            return True
        if self.torrent.timeoutaction == 1:
            self.torrent.incDecPriority(-1, self.listid)
            if willberesumed:
                return False
            self.parent.procQUEUE([self.listid])
            return True

    def onTrackerOK(self, tracker, external):
        if self.torrent.multitracker and not external:
            self.torrent.setDefaultTracker(tracker)
        self.firsterrortracker = 0
        self.timeoutscanceled = False
        if self.torrent.message and self.isTrackerCnxError(self.torrent.message[self.utility.timestamplen + self.utility.messageheaderlen:]):
            self.torrent.clearMessage(self.listid)

    def countWorkingTimeLeft(self, t, sizedone):
        # Working time left
        if self.meandownrate > 0:
            workingtimeleft = max(self.workingtimeleft - t + self.workingtimelastcheck, 0)
            newworkingtimeleft = (self.torrent.requestedsize - sizedone) / (self.meandownrate * 1024)
            if newworkingtimeleft > 86400:
                delta = newworkingtimeleft / 20
            elif newworkingtimeleft > 60:
                delta = newworkingtimeleft / 10
            elif newworkingtimeleft > 10:
                delta = 2
            else:
                delta = 1
            if abs(workingtimeleft - newworkingtimeleft) > delta:
                # If timer value deviates too much from theoretical value, reset it to theoretical value
                workingtimeleft = newworkingtimeleft
        else:
            # Set to 100 days (= infinite)
            workingtimeleft = 8640000
        self.workingtimeleft = workingtimeleft
        self.workingtimelastcheck = t

    def countSeedingTimes(self, t):
        # Seeding times
        if self.seedingtimer:
            self.torrent.seedingtime = self.getCurrentSeedingTime(t)
        if self.totalseedingtimer:
            self.torrent.totalseedingtime = self.getCurrentTotalSeedingTime(t)
        self.countSeedingTimeLeft(t)

    def countSeedingTimeLeft(self, t):
        # Seeding time left
        if self.torrent.uploadoption == "1" or self.torrent.uploadoption == "3":
            timeshift = self.torrent.uploadtimemax - self.torrent.seedingtime
            # Correct seeding times shifting because of check rate
            if timeshift > 0:
                self.olduploadtimemax = self.torrent.uploadtimemax
            elif self.torrent.uploadtimemax >= self.olduploadtimemax:
                self.torrent.seedingtime = self.torrent.uploadtimemax
                self.torrent.totalseedingtime += timeshift
            seedingtimeleft_time = max(timeshift, 0)
        if self.torrent.uploadoption == "2" or self.torrent.uploadoption == "3":
            if self.torrent.ratioIsReached():
                seedingtimeleft_ratio = 0
            else:
                if self.meanuprate > 0:
                    seedingtimeleft = max(self.torrent.seedingtimeleft - t + self.seedingtimelastcheck, 0)
                    down, up = self.getDownUpSize()
                    if down <= 0:
                        down = self.torrent.requestedsize
                    newseedingtimeleft = max((int(self.torrent.uploadratio) * down / 100 - up) / (self.meanuprate * 1024), 0)
                    if newseedingtimeleft > 86400:
                        delta = newseedingtimeleft / 20
                    elif newseedingtimeleft > 60:
                        delta = newseedingtimeleft / 10
                    elif newseedingtimeleft > 10:
                        delta = 2
                    else:
                        delta = 1
                    if abs(seedingtimeleft - newseedingtimeleft) > delta:
                        # If timer value deviates too much from theoretical value, reset it to theoretical value
                        seedingtimeleft = newseedingtimeleft
                else:
                    # Set to 100 days (= infinite)
                    seedingtimeleft = 8640000
                seedingtimeleft_ratio = seedingtimeleft
        if self.torrent.uploadoption == "1":
            self.torrent.seedingtimeleft = seedingtimeleft_time
        elif self.torrent.uploadoption == "2":
            self.torrent.seedingtimeleft = seedingtimeleft_ratio
        elif self.torrent.uploadoption == "3":
            if seedingtimeleft_time <= seedingtimeleft_ratio:
                self.torrent.seedingtimeleft = seedingtimeleft_time
            else:
                self.torrent.seedingtimeleft = seedingtimeleft_ratio
        self.seedingtimelastcheck = t

    def startSeedingTimers(self, resettotalseeding = True):
        now = clock()
        if self.utility.seedingtimecount == "0" or self.torrent.uploadoption == "1" \
           or self.torrent.uploadoption == "3":
            self.startSeedingTimer(now)
        if resettotalseeding:
            self.startTotalSeedingTimer(now)

    def startSeedingTimer(self, t):
        self.seedingtimer = 1
        self.seedingtimerreset = self.seedingtimelastcheck = t
        self.lastseedingtime = self.torrent.seedingtime

    def startTotalSeedingTimer(self, t):
        self.totalseedingtimer = 1
        self.totalseedingtimerreset = t
        self.lasttotalseedingtime = self.torrent.totalseedingtime

    def stopSeedingTimers(self, resettotalseeding = True):
        now = clock()
        if self.seedingtimer:
            self.stopSeedingTimer(now)
        if self.totalseedingtimer and resettotalseeding:
            self.stopTotalSeedingTimer(now)

    def stopSeedingTimer(self, t):
        self.torrent.seedingtime = self.getCurrentSeedingTime(t)
        self.seedingtimer = 0

    def stopTotalSeedingTimer(self, t):
        self.torrent.totalseedingtime = self.getCurrentTotalSeedingTime(t)
        self.totalseedingtimer = 0

    def getCurrentSeedingTime(self, t):
        return self.lastseedingtime + t - self.seedingtimerreset

    def getCurrentTotalSeedingTime(self, t):
        return self.lasttotalseedingtime + t - self.totalseedingtimerreset

    def initForceSeeding(self):
        self.normalseedingover = True
        if not self.torrent.scraping:
            self.torrent.scrapeisfresh = False
            self.torrent.lastgetscrape = 0
        etarank = self.getRankfromID(8)
        if etarank != -1:
            self.list.SetStringItem(self.listid, etarank, self.localize('etaS') + " " + self.localize('tillcc'))

    def checkForTerminating(self, queueback = False):
        if self.forceseeding:
            if queueback:
                # Torrent is limited and must not seed
                self.onLimitedTorrent()
            else:
                self.initForceSeeding()
        else:
            self.terminateUpload()

    def terminateUpload(self):
        ####################################################
        # change :  5:Progress 6:BT Status 13:Message
        # untouch : 4:Title 7:Priority 9:Size 12:%U/DSize
        #           18:DownloadSize 19:UploadSize 21:torrentname
        #           22:seeding option
        # clear :   8:ETA 10:DLSpeed 11:ULspeed
        #           14:#seed 15:#peer 16:#copies 17:peer avg
        #           20:total speed
        #####################################################
        self.btstatus = 'finished'
        self.progress = 100.
        self.torrent.seedingtimeleft = 0
        self.colour = self.utility.colnoeng
        self.updateColour()

        self.parent.procFINISHED(self.listid, self.onlycheck)

    def onCompletedEvent(self, initphase = True):
        ####################################################
        # change :  5:Progress 6:BT Status 8:ETA 12:%U/DSize
        #           13:Message
        # untouch : 4:Title 7:Priority 9:Size
        #           18:DownloadSize 19:UploadSize 11:ULspeed
        #           14:#seed 15:#peer 16:#copies 17:peer avg
        #           20:total speed 21:torrentname
        #           22:seeding option
        # clear :   10:DLSpeed  
        #####################################################
        if self.torrent.status == 'pause' or self.torrent.status == 'onhold':
            return
        self.fin = True
        self.btstatus = 'completedseeding'
        self.progress = 100.
        self.normalseedingover = False
        self.torrent.complete = True

        rank = self.getRankfromID(5)
        if rank != -1:
            self.list.SetStringItem(self.listid, rank, self.formatedProgress())
        if not self.onlycheck:
            rank = self.getRankfromID(8)
            if rank != -1:
                if self.torrent.uploadoption == "0":
                    self.list.SetStringItem(self.listid, rank, self.localize('etaS') + "     oo")
                elif self.torrent.uploadoption == "1":
                    self.list.SetStringItem(self.listid, rank, self.localize('etaS') + " " + self.utility.eta_value(self.torrent.seedingtimeleft))
                else:
                    self.list.SetStringItem(self.listid, rank, self.localize('etaS') + "      ?")
            rank = self.getRankfromID(10)
            if rank != -1:
                self.list.SetStringItem(self.listid, rank, self.utility.formatedRate(None, self.torrent.prioritizedown)[0])
        rank = self.getRankfromID(12)
        if rank != -1:
            self.list.SetStringItem(self.listid, rank, self.shareratiotext)

        # Update detail windows
        if self.infowin:
            if not self.onlycheck:
                self.infowin.downrate.SetLabel(self.utility.rateWithUnit('', self.torrent.prioritizedown))
            if not self.torrent.singlefile and not self.torrent.unwantedfiles:
                # Force display of progress of all files to Done
                donestr = self.localize('done')
                progresscol = 3 + self.infowin.originalfilenamesshown
                for x in xrange(self.torrent.filesnumber):
                    self.infowin.filelist.SetStringItem(x, progresscol, donestr)
            self.infowin.totalprogress.SetLabel(self.formatedProgress())
        if self.detailwin:
            if not self.onlycheck:
                self.detailwin.downrate.SetLabel(self.utility.rateWithUnit('', self.torrent.prioritizedown))
            self.detailwin.totalprogress.SetLabel(self.formatedProgress())

        if not self.parent.procCOMPLETED(self.listid, self.alreadycomplete, self.onlycheck, initphase) \
           and self.torrent.status != 'finished' and self.torrent.status != 'queue' and not self.onlycheck:
            # Display seeding status
            rank = self.getRankfromID(6)
            if rank != -1:
                self.list.SetStringItem(self.listid, rank, self.localize(self.btstatus))
            if self.infowin:
                self.infowin.status.SetLabel(self.localize(self.btstatus))
            if self.detailwin:
                self.detailwin.status.SetLabel(self.localize(self.btstatus))
            # Start seeding
            self.startSeedingTimers()

    def onFailEvent(self):
        self.parent.procSTOP([self.listid], torrentfailed = True)

    def onLimitedTorrent(self):
        self.parent.procQUEUE([self.listid])

    def chooseFile(self, default, size, saveas, dir):
        if saveas != '':       
            self.filename = default
            if self.olddownsize > 0:
                self.shareratio = float(self.oldupsize) / self.olddownsize * 100
            else:
                if self.torrent.requestedsize <= 0 and not self.torrent.magnet:
                    self.shareratio = 999.9
                elif self.progress <= 0:
                    self.shareratio = 0.
                else:
                    self.shareratio = float(self.oldupsize) / self.torrent.requestedsize / self.progress * 10000
            self.shareratiotext = str(.1 * int(10 * self.shareratio)) + "%"
            return self.utility.completePath(saveas)
        return None
    
    def start(self):
        self.torrent.displayStatusOnly(self.listid, 'waiting')
        self.fastresume = self.fastresume and (self.torrent.checkedonce or int(self.abcparams['fastresumefirst']))
        self.shutdownflag.clear()
        thread = Thread(target = self.startBT)
        thread.daemon = False
        thread.start()

    def startBT(self):
        limitedtorrent = False
        if self.onlycheck:
            checkover = False

        try:
            while True:
                try:
                    config = parse_params(self.btparams, {})
                except ValueError, e:
                    self.error(self.localize('error') + ' : ' + exceptionArgsToString(e) + self.localize('invalidbtconfig'))
                    break
                if not config:
                    self.error(self.localize('error') + ' : ' + self.localize('invalidbtconfig'))
                    break

                myid = createPeerID()

                rawserver = RawServer(self.doneflag, config['timeout_check_interval'],
                                      config['timeout'], ipv6_enable = config['ipv6_enabled'],
                                      failfunc = self.error, errorfunc = self.error)

                if self.onlycheck:
                    upnp_type = listen_port = None
                else:
                    upnp_type = UPnP_test(config['upnp_nat_access'])
                    try:
                        listen_port = rawserver.find_and_bind(config['minport'], config['maxport'], config['bind'],
                                                              ipv6_socket_style = config['ipv6_binds_v4'], upnp = upnp_type,
                                                              randomizer = config['random_port'])
                    except socketerror, e:
                        self.error(self.localize('cantlisten') + " - " + exceptionArgsToString(e))
                        break

                response = get_response(config['responsefile'], self.error)
                if not response:
                    break

                if self.torrent.exttracker:
                    extannounce = self.torrent.exttrackerurl
                else:
                    extannounce = None

                dow = BT1Download(self.updateStatus, self.completed, self.error, self.error, self.doneflag,
                                  config, response, self.torrent.infohash, myid, rawserver, listen_port, self.dht,
                                  extann = extannounce, trackerokfunc = self.trackerOK,
                                  endallocfunc = self.endAllocation, rejectexceptions = self.parent.rejectexceptions,
                                  filesnamefunc = self.torrent.getFileNames,
                                  resetcompletefunc = self.torrent.resetCompleteStatus,
                                  metadatacompletefunc = self.metadataComplete, reseed = self.reseed,
                                  checkfreediskspacefunc = self.checkFreeDiskSpace)

                self.dow = dow

                # Keep or delete cache data according to fastresume user setting
                if not self.fastresume:
                    dow.appdataobj.deleteTorrentData(self.torrent.infohash)

                if not dow.saveAs(self.chooseFile):
                    break
                if not dow.initFiles(FILESEM):
                    dow.shutdown()
                    break
                if not self.torrent.magnet:
                    self.torrent.checkedonce = True

                if self.onlycheck:
                    # Manual data checking
                    # Get files statistics
                    if not dow.startEngine():
                        dow.shutdown()
                        break
                    dow._init_stats()
                    self.reportProgress(dow.statistics.update())

                    if self.torrent.unwantedfiles:
                        self.torrent.checkComplete()

                    if not self.btcomplete:
                        # Retrieve and display checked progress
                        obtained, desired = dow.storagewrapper.get_stats()
                        if desired > 0:
                            self.torrent.progress = float(obtained) / desired * 100
                        else:
                            self.torrent.progress = 100.
                    elif self.torrent.unwantedfiles:
                        self.completed(False)

                    # Write file progress to disk
                    self.torrent.writeFileProgress(update = False)

                    # Save resume data and stop torrent
                    dow.shutdown()

                    if not self.torrent.complete:
                        self.parent.invokeLater(self.parent.displayCheckResult, [self.torrent])

                    checkover = True
                    break

                # Fix complete status if needed
                self.torrent.applyRealCompleteStatus()
                if self.auto and self.torrent.isOverSeedingLimit():
                    # The real complete status makes the torrent limited in connections (only for automatically started torrents)
                    limitedtorrent = True
                    break

                if not dow.startEngine():
                    dow.shutdown()
                    break

                if self.sbwaitingforleechers:
                    self.sbwaitingforleechers = clock()
                else:
                    # Starting torrents are not immediately added to torrentalive
                    # They will be added in next run of cyclicalTasks with correct index
                    # according to priority.
                    # Reset last starting time for the URM
                    self.parent.urmstartingtime = clock()

                dow.startRerequester()

                if self.dht and not self.torrent.private:
                    dow.startDHTRerequester()

                dow.autoStats()

                if not dow.am_I_finished():
                    self.updateStatus(activity = 'connectingtopeers')

                # Set the spew flag if the detail window is shown
                if self.torrent.detailwin is not None:
                    dow.spewflag.set()

                self.filespriochangeallowed = True

                exception = rawserver.listen_forever(dow.getPortHandler())

                dow.shutdown(exception = exception)
                break
        except:
            data = StringIO()
            print_exc(file = data)
            sys.stderr.write(data.getvalue())

        if self.onlycheck:
            if not self.stoppedonrequest:
                if checkover:
                    if not self.btcomplete:
                        self.invokeLater(self.stopEngine)
                else:
                    self.failed()
        else:
            try:
                rawserver.shutdown()
            except:
                pass
            if limitedtorrent:
                self.handleLimitedTorrent()
            if not self.stoppedonrequest:
                self.failed()

        self.shutdownflag.set()

    def stop(self):
        self.stopSeedingTimers()
        self.stoppedonrequest = True
        self.stopBT()
        self.stopEngine()

    def stopBT(self):
        self.doneflag.set()
        self.shutdownflag.wait()

    def stopEngine(self):
        DELEVT_INVOKE(self)
        self.dow = None
        self.torrent.abcengine = None

    def pause(self):
        if self.dow and self.dow.Pause():
            self.noupdateGUI = True
            self.colour = self.utility.colnoeng
            return True
        self.noupdateGUI = False
        return False

    def unpause(self, reallyrun = True):
        self.noupdateGUI = not reallyrun
        if self.dow:
            self.dow.Unpause()
            if reallyrun and self.btcomplete and not self.fin:
                # BT generated the completed event but it could not be processed because of pausing
                self.onCompletedEvent()

    def updateColour(self):
        self.list.SetItemTextColour(self.listid, self.colour)

    def endAllocation(self):
        self.torrent.totallyallocated = True

    def metadataComplete(self, metadata):
        self.invokeLater(self.onMetadataComplete, [metadata])

    def onMetadataComplete(self, metadata):
        if sha1(metadata.viewSlice()).digest() != self.torrent.infohash:
            # Invalid metadata
            self.torrent.displayMessage(strftime(self.utility.timestamp) + self.utility.messageheader
                                        + self.localize('error') + ' : '
                                        + self.localize('failedinvalidtorrent_short'), self.listid)
            self.dow.encoder.onNoMetadata()
            return

        try:
            metadatadict = bdecode(metadata[:], sloppy = 1)
        except:
            # Invalid metadata
            self.torrent.displayMessage(strftime(self.utility.timestamp) + self.utility.messageheader
                                        + self.localize('error') + ' : '
                                        + self.localize('failedinvalidtorrent_short'), self.listid)
            self.failed()
            return

        try:
            newtorrentname = metadatadict.get('name.utf-8')
            if newtorrentname is None:
                newtorrentname = metadatadict['name']
        except:
            # Invalid metadata
            self.torrent.displayMessage(strftime(self.utility.timestamp) + self.utility.messageheader
                                        + self.localize('error') + ' : '
                                        + self.localize('failedinvalidtorrent_short'), self.listid)
            self.failed()
            return
        newtorrentname = self.utility.decodeString(newtorrentname)
        fixednewtorrentname = self.utility.fixWindowsName(newtorrentname)
        if fixednewtorrentname:
            newtorrentfilename = fixednewtorrentname
        else:
            newtorrentfilename = newtorrentname

        # Build new torrent
        params = {'addclosestdhtnodes': 1, 'info': metadatadict}
        # Trackers
        announce = self.dow.response.get('announce')
        if announce is not None:
            params['real_url'] = announce
        else:
            announcelist = self.dow.response.get('announce-list')
            if announcelist is not None:
                params['real_announce_list'] = announcelist
        # New complete torrent metadata
        btmetafile = make_meta_data('', '', self.dht, params)[0]
        newtorrentfilepath = self.utility.findUniqueFileName(path.join(self.utility.datapath, 'torrent', newtorrentfilename + '.torrent'))
        if newtorrentfilepath is None:
            return
        # Write new torrent file
        try:
            f = open(newtorrentfilepath, "wb")
            f.write(btmetafile)
            f.close()
        except:
            try:
                f.close()
            except:
                pass
            dfmessage = strftime(self.utility.timestamp) + self.utility.messageheader + self.localize('error') + ' : ' + self.localize('diskfull')
            drive = path.splitdrive(self.utility.completePath(self.dest))[0]
            if sys.platform == 'win32' and drive:
                message += ' (' + drive + ')'
            self.torrent.displayMessage(dfmessage, self.listid)
            self.failed()
            return

        # Replace torrent metadata with new ones
        oldtorrentfile = self.torrent.src
        success, mesg = self.parent.addNewProc(newtorrentfilepath, newtorrentfilename, False, replace = self.listid)
        if not success:
            # Torrent could not be loaded
            self.failed()
            return

        # Metadata are valid and were successfully loaded

        # Remove old torrent file
        try:
            remove(oldtorrentfile)
        except:
            pass

        # The torrent must be stopped because manual modifications in content or destination are needed
        if mesg == 'Stop torrent':
            self.failed()
            return

        # Check if torrent is private without trackers
        if self.torrent.isPrivateWithoutTrackers():
            self.invokeLater(self.parent.procSTOP, [[self.listid]])
            return

        self.btstatus = 'waiting'
        self.restarted = True

        self.dow.updateStorage(self.chooseFile, FILESEM, self.updateStorageComplete)

    def updateStorageComplete(self):
        self.btstatus = 'connectingtopeers'
        self.filespriochangeallowed = True
        self.torrent.checkedonce = True

    def checkFreeDiskSpace(self, onstart = False):
        alloctype = self.abcparams['alloc_type']
        if not self.torrent.magnet and not self.torrent.complete \
           and (alloctype == '1' and not onstart or alloctype == '2') \
           and not self.parent.diskFullWatchdog(self.torrent, onstart = onstart):
            return False
        return True
