##############################################################################
# Module : template.py
# Author : Old King Cole
# Date   : 01/29/2005
#
# Description : Parameters templates for torrents
#
##############################################################################
from filemanager import TemplateListFileManager


##############################################################
# Class : Template
##############################################################
class Template:
    def __init__(self, description, hidden,
                 templabelswitch, shortlabel,
                 tempdownloadswitch, maxdownrate, prioritizedown,
                 tempuploadswitch, maxupload, maxuprate, prioritizeup,
                 tempseedingswitch, uploadoption, uploadtimeh, uploadtimem, uploadratio,
                 temptimeoutswitch, timeoutswitch, timeoutaction, timeoutwork, timeoutworkdown,
                 timeoutworktrack, timeoutseed, timeoutseedup, timeoutseedtrack, timeouttracker,
                 timeoutdownload, timeoutupload, tempmoveswitch, movefolder,
                 ruletrigrank, ruleactive,
                 ruletracker, ruletrackersw, ruletorrent, ruletorrentsw,
                 ruletorrentfile, ruletorrentfilesw, rulereceived, rulereceivedsw):
        self.description = description
        self.hidden = hidden
        self.templabelswitch = templabelswitch
        self.shortlabel = shortlabel
        self.tempdownloadswitch = tempdownloadswitch
        self.maxdownrate = maxdownrate
        self.prioritizedown = prioritizedown
        self.tempuploadswitch = tempuploadswitch
        self.maxupload = maxupload
        self.maxuprate = maxuprate
        self.prioritizeup = prioritizeup
        self.tempseedingswitch = tempseedingswitch
        self.uploadoption = uploadoption[-1]
        self.uploadtillcopycomplete = (int(uploadoption) / 10 == 1)
        self.uploadtimeh = uploadtimeh
        self.uploadtimem = uploadtimem
        self.uploadratio = uploadratio
        self.temptimeoutswitch = temptimeoutswitch
        self.timeoutswitch = timeoutswitch
        self.timeoutaction = timeoutaction
        self.timeoutwork = timeoutwork
        self.timeoutworkdown = timeoutworkdown
        self.timeoutworktrack = timeoutworktrack
        self.timeoutseed = timeoutseed
        self.timeoutseedup = timeoutseedup
        self.timeoutseedtrack = timeoutseedtrack
        self.timeouttracker = timeouttracker
        self.timeoutdownload = timeoutdownload
        self.timeoutupload = timeoutupload
        self.tempmoveswitch = tempmoveswitch
        self.movefolder = movefolder
        self.ruletrigrank = ruletrigrank
        self.ruleactive = ruleactive
        self.ruletracker = ruletracker
        self.ruletrackersw = ruletrackersw
        self.ruletorrent = ruletorrent
        self.ruletorrentsw = ruletorrentsw
        self.ruletorrentfile = ruletorrentfile
        self.ruletorrentfilesw = ruletorrentfilesw
        self.rulereceived = rulereceived
        self.rulereceivedsw = rulereceivedsw
        self.conlimiters = []

    def isTriggerRuleActive(self):
        return self.ruleactive \
               and (self.ruletracker != '' or self.ruletorrent != '' or self.ruletorrentfile != '' or self.rulereceived != '')


##############################################################
# Class : TemplateManager
##############################################################
class TemplateManager:
    def __init__(self, parent):
        self.parent = parent
        self.utility = parent.utility
        self.templates = {}
        self.tempbyindex = []
        self.tempbytrigrank = []

        templatelistfile = TemplateListFileManager(self)
        templatelistfile.open()
        index = -1
        while True:
            temp = templatelistfile.readList()
            if temp is None:
                break
            if len(temp) != 41:
                continue
            index += 1
            if temp[22] == '-1':
                temp[22] = str(index)
            self.templates[temp[0]] = Template(temp[1], int(temp[21]), int(temp[2]), temp[3], int(temp[18]), int(temp[19]), int(temp[39]),
                                               int(temp[4]), int(temp[5]), int(temp[6]), int(temp[40]), int(temp[20]), temp[7], int(temp[8]),
                                               int(temp[9]), int(temp[10]), int(temp[11]), int(temp[12]), int(temp[38]), int(temp[32]),
                                               int(temp[33]), int(temp[34]), int(temp[35]), int(temp[36]), int(temp[37]), temp[13], temp[14],
                                               temp[15], int(temp[16]), temp[17], int(temp[22]), int(temp[23]), temp[24], int(temp[25]),
                                               temp[26], int(temp[27]), temp[28], int(temp[29]), temp[30], int(temp[31]))
            self.tempbyindex.append(temp[0])
        templatelistfile.close()
        # Check if default template exists
        nodefault = False
        if self.utility.abcparams['templatedefault'] not in self.templates:
            if 'default' not in self.templates:
                self.createFirstDefaultTemplate()
            self.utility.abcparams['templatedefault'] = 'default'
            nodefault = True

        trigrankandkey = [(self.templates[key].ruletrigrank, key) for key in self.templates.keys()]
        trigrankandkey.sort()
        self.tempbytrigrank = [t[1] for t in trigrankandkey]

        if nodefault or templatelistfile.oldformat:
            templatelistfile.open('w+')
            templatelistfile.writeList(self.templates, self.tempbyindex, self.tempbytrigrank)
            templatelistfile.close()

    def createFirstDefaultTemplate(self):
        self.templates['default'] = Template('Default template',
                                             0,
                                             1,
                                             'def',
                                             1,
                                             0,
                                             0,
                                             1,
                                             4,
                                             0,
                                             0,
                                             1,
                                             '0',
                                             0,
                                             30,
                                             50,
                                             1,
                                             1,
                                             0,
                                             1,
                                             1,
                                             1,
                                             1,
                                             1,
                                             1,
                                             '60',
                                             '60',
                                             '1',
                                             1,
                                             '',
                                             len(self.templates),
                                             0,
                                             '',
                                             0,
                                             '',
                                             0,
                                             '',
                                             0,
                                             '',
                                             0
                                             )
        self.tempbyindex.append('default')
        self.tempbytrigrank.append('default')
    
    def createTemplate(self, dico, tempbyindex, tempbytrigrank, name, default):
        deftemp = dico[default]
        dico[name] = Template(deftemp.description,
                              deftemp.hidden,
                              deftemp.templabelswitch,
                              deftemp.shortlabel,
                              deftemp.tempdownloadswitch,
                              deftemp.maxdownrate,
                              deftemp.prioritizedown,
                              deftemp.tempuploadswitch,
                              deftemp.maxupload,
                              deftemp.maxuprate,
                              deftemp.prioritizeup,
                              deftemp.tempseedingswitch,
                              deftemp.uploadoption,
                              deftemp.uploadtimeh,
                              deftemp.uploadtimem,
                              deftemp.uploadratio,
                              deftemp.temptimeoutswitch,
                              deftemp.timeoutswitch,
                              deftemp.timeoutaction,
                              deftemp.timeoutwork,
                              deftemp.timeoutworkdown,
                              deftemp.timeoutworktrack,
                              deftemp.timeoutseed,
                              deftemp.timeoutseedup,
                              deftemp.timeoutseedtrack,
                              deftemp.timeouttracker,
                              deftemp.timeoutdownload,
                              deftemp.timeoutupload,
                              deftemp.tempmoveswitch,
                              deftemp.movefolder,
                              len(self.templates),
                              0,
                              '',
                              0,
                              '',
                              0,
                              '',
                              0,
                              '',
                              0
                              )
        tempbyindex.append(name)
        tempbytrigrank.append(name)
